@@ -1,11 +1,13 @@
 use std::borrow::Cow;
 use std::sync::Arc;
 
+use polars_utils::no_call_const;
+
 use crate::array::static_array::{ParameterFreeDtypeStaticArray, StaticArray};
 use crate::array::{
     Array, BinaryArray, BinaryViewArray, BooleanArray, FixedSizeListArray, ListArray,
     MutableBinaryArray, MutableBinaryValuesArray, MutableBinaryViewArray, PrimitiveArray,
-    Utf8Array, Utf8ViewArray,
+    StructArray, Utf8Array, Utf8ViewArray,
 };
 use crate::bitmap::Bitmap;
 use crate::datatypes::ArrowDataType;
@@ -901,7 +903,39 @@ impl<T: AsArray> ArrayFromIterDtype<Option<T>> for ListArray<i64> {
         iter: I,
     ) -> Result<Self, E> {
         let iter_values = iter.into_iter().collect::<Result<Vec<_>, E>>()?;
-        Ok(Self::arr_from_iter_with_dtype(dtype, iter_values))
+        let mut builder = AnonymousListArrayBuilder::new(iter_values.len());
+        for arr in &iter_values {
+            builder.push_opt(arr.as_ref().map(|a| a.as_array()));
+        }
+        let inner = dtype
+            .inner_dtype()
+            .expect("expected nested type in ListArray collect");
+        Ok(builder
+            .finish(Some(&inner.underlying_physical_type()))
+            .unwrap())
+    }
+}
+
+impl<T: AsArray> ArrayFromIter<Option<T>> for ListArray<i64> {
+    fn arr_from_iter<I: IntoIterator<Item = Option<T>>>(iter: I) -> Self {
+        let iter = iter.into_iter();
+        let iter_values: Vec<Option<T>> = iter.into_iter().collect();
+        let mut builder = AnonymousListArrayBuilder::new(iter_values.len());
+        for arr in &iter_values {
+            builder.push_opt(arr.as_ref().map(|a| a.as_array()));
+        }
+        builder.finish(None).unwrap()
+    }
+
+    fn try_arr_from_iter<E, I: IntoIterator<Item = Result<Option<T>, E>>>(
+        iter: I,
+    ) -> Result<Self, E> {
+        let iter_values = iter.into_iter().collect::<Result<Vec<_>, E>>()?;
+        let mut builder = AnonymousListArrayBuilder::new(iter_values.len());
+        for arr in &iter_values {
+            builder.push_opt(arr.as_ref().map(|a| a.as_array()));
+        }
+        Ok(builder.finish(None).unwrap())
     }
 }
 
@@ -984,3 +1018,57 @@ impl ArrayFromIterDtype<Option<Box<dyn Array>>> for FixedSizeListArray {
         Ok(Self::arr_from_iter_with_dtype(dtype, iter_values))
     }
 }
+
+impl ArrayFromIter<Option<()>> for StructArray {
+    fn arr_from_iter<I: IntoIterator<Item = Option<()>>>(_iter: I) -> Self {
+        no_call_const!()
+    }
+
+    fn try_arr_from_iter<E, I: IntoIterator<Item = Result<Option<()>, E>>>(
+        _iter: I,
+    ) -> Result<Self, E> {
+        no_call_const!()
+    }
+}
+
+impl ArrayFromIter<()> for StructArray {
+    fn arr_from_iter<I: IntoIterator<Item = ()>>(_iter: I) -> Self {
+        no_call_const!()
+    }
+
+    fn try_arr_from_iter<E, I: IntoIterator<Item = Result<(), E>>>(_iter: I) -> Result<Self, E> {
+        no_call_const!()
+    }
+}
+
+impl ArrayFromIterDtype<()> for StructArray {
+    fn arr_from_iter_with_dtype<I: IntoIterator<Item = ()>>(
+        _dtype: ArrowDataType,
+        _iter: I,
+    ) -> Self {
+        no_call_const!()
+    }
+
+    fn try_arr_from_iter_with_dtype<E, I: IntoIterator<Item = Result<(), E>>>(
+        _dtype: ArrowDataType,
+        _iter: I,
+    ) -> Result<Self, E> {
+        no_call_const!()
+    }
+}
+
+impl ArrayFromIterDtype<Option<()>> for StructArray {
+    fn arr_from_iter_with_dtype<I: IntoIterator<Item = Option<()>>>(
+        _dtype: ArrowDataType,
+        _iter: I,
+    ) -> Self {
+        no_call_const!()
+    }
+
+    fn try_arr_from_iter_with_dtype<E, I: IntoIterator<Item = Result<Option<()>, E>>>(
+        _dtype: ArrowDataType,
+        _iter: I,
+    ) -> Result<Self, E> {
+        no_call_const!()
+    }
+}
