@@ -1,12 +1,14 @@
 use std::ops::Deref;
 use std::sync::atomic::{AtomicU64, Ordering};
-use std::sync::Arc;
+use std::sync::{Arc, OnceLock};
 
 use either::Either;
+use parking_lot::RwLockUpgradableReadGuard;
 use polars_error::{polars_bail, PolarsResult};
 
 use super::utils::{count_zeros, fmt, get_bit, get_bit_unchecked, BitChunk, BitChunks, BitmapIter};
-use super::{chunk_iter_to_vec, IntoIter, MutableBitmap};
+use super::{chunk_iter_to_vec, intersects_with, num_intersections_with, IntoIter, MutableBitmap};
+use crate::array::Splitable;
 use crate::bitmap::aligned::AlignedBitmapSlice;
 use crate::bitmap::iterator::{
     FastU32BitmapIter, FastU56BitmapIter, FastU64BitmapIter, TrueIdxIter,
@@ -63,6 +65,11 @@ pub struct Bitmap {
     unset_bit_count_cache: AtomicU64,
 }
 
+#[inline(always)]
+fn has_cached_unset_bit_count(ubcc: u64) -> bool {
+    ubcc >> 63 == 0
+}
+
 impl Clone for Bitmap {
     fn clone(&self) -> Self {
         Self {
@@ -117,7 +124,7 @@ impl Bitmap {
             length,
             offset: 0,
             bytes: Arc::new(bytes.into()),
-            unset_bit_count_cache: AtomicU64::new(UNKNOWN_BIT_COUNT),
+            unset_bit_count_cache: AtomicU64::new(if length == 0 { 0 } else { UNKNOWN_BIT_COUNT }),
         })
     }
 
@@ -216,15 +223,12 @@ impl Bitmap {
     /// This function counts the number of unset bits if it is not already
     /// computed. Repeated calls use the cached bitcount.
     pub fn unset_bits(&self) -> usize {
-        let cache = self.unset_bit_count_cache.load(Ordering::Relaxed);
-        if cache >> 63 != 0 {
+        self.lazy_unset_bits().unwrap_or_else(|| {
             let zeros = count_zeros(&self.bytes, self.offset, self.length);
             self.unset_bit_count_cache
                 .store(zeros as u64, Ordering::Relaxed);
             zeros
-        } else {
-            cache as usize
-        }
+        })
     }
 
     /// Returns the number of unset bits on this [`Bitmap`] if it is known.
@@ -232,11 +236,7 @@ impl Bitmap {
     /// Guaranteed to be `<= self.len()`.
     pub fn lazy_unset_bits(&self) -> Option<usize> {
         let cache = self.unset_bit_count_cache.load(Ordering::Relaxed);
-        if cache >> 63 != 0 {
-            None
-        } else {
-            Some(cache as usize)
-        }
+        has_cached_unset_bit_count(cache).then_some(cache as usize)
     }
 
     /// Updates the count of the number of set bits on this [`Bitmap`].
@@ -286,7 +286,7 @@ impl Bitmap {
             return;
         }
 
-        if *unset_bit_count_cache >> 63 == 0 {
+        if has_cached_unset_bit_count(*unset_bit_count_cache) {
             // If we keep all but a small portion of the array it is worth
             // doing an eager re-count since we can reuse the old count via the
             // inclusion-exclusion principle.
@@ -400,7 +400,58 @@ impl Bitmap {
     /// Initializes an new [`Bitmap`] filled with unset values.
     #[inline]
     pub fn new_zeroed(length: usize) -> Self {
-        Self::new_with_value(false, length)
+        // There are quite some situations where we just want a zeroed out Bitmap, since that would
+        // constantly need to reallocate we make a static that contains the largest allocation.
+        // Then, we can just take an Arc::clone of that slice everytime or grow it if needed.
+        static GLOBAL_ZERO_BYTES: OnceLock<parking_lot::RwLock<Arc<Bytes<u8>>>> = OnceLock::new();
+
+        let rwlock_zero_bytes = GLOBAL_ZERO_BYTES.get_or_init(|| {
+            let byte_length = length.div_ceil(8).next_power_of_two();
+            parking_lot::RwLock::new(Arc::new(Bytes::from(vec![0; byte_length])))
+        });
+
+        let unset_bit_count_cache = AtomicU64::new(length as u64);
+
+        let zero_bytes = rwlock_zero_bytes.upgradable_read();
+        if zero_bytes.len() * 8 >= length {
+            let bytes = zero_bytes.clone();
+            return Bitmap {
+                bytes,
+                offset: 0,
+                length,
+                unset_bit_count_cache,
+            };
+        }
+
+        let mut zero_bytes = RwLockUpgradableReadGuard::upgrade(zero_bytes);
+
+        // Race Condition:
+        // By the time we got here, another Guard could have been upgraded, and the buffer
+        // could have been expanded already. So we want to check again whether we cannot just take
+        // that buffer.
+        if zero_bytes.len() * 8 >= length {
+            let bytes = zero_bytes.clone();
+            return Bitmap {
+                bytes,
+                offset: 0,
+                length,
+                unset_bit_count_cache,
+            };
+        }
+
+        // Let do exponential increases so that we are not constantly allocating new
+        // buffers.
+        let byte_length = length.div_ceil(8).next_power_of_two();
+
+        let bytes = Arc::new(Bytes::from(vec![0; byte_length]));
+        *zero_bytes = bytes.clone();
+
+        Bitmap {
+            bytes,
+            offset: 0,
+            length,
+            unset_bit_count_cache,
+        }
     }
 
     /// Initializes an new [`Bitmap`] filled with the given value.
@@ -474,6 +525,36 @@ impl Bitmap {
             unset_bit_count_cache,
         }
     }
+
+    /// Checks whether two [`Bitmap`]s have shared set bits.
+    ///
+    /// This is an optimized version of `(self & other) != 0000..`.
+    pub fn intersects_with(&self, other: &Self) -> bool {
+        intersects_with(self, other)
+    }
+
+    /// Calculates the number of shared set bits between two [`Bitmap`]s.
+    pub fn num_intersections_with(&self, other: &Self) -> usize {
+        num_intersections_with(self, other)
+    }
+
+    /// Select between `truthy` and `falsy` based on `self`.
+    ///
+    /// This essentially performs:
+    ///
+    /// `out[i] = if self[i] { truthy[i] } else { falsy[i] }`
+    pub fn select(&self, truthy: &Self, falsy: &Self) -> Self {
+        super::bitmap_ops::select(self, truthy, falsy)
+    }
+
+    /// Select between `truthy` and constant `falsy` based on `self`.
+    ///
+    /// This essentially performs:
+    ///
+    /// `out[i] = if self[i] { truthy[i] } else { falsy }`
+    pub fn select_constant(&self, truthy: &Self, falsy: bool) -> Self {
+        super::bitmap_ops::select_constant(self, truthy, falsy)
+    }
 }
 
 impl<P: AsRef<[bool]>> From<P> for Bitmap {
@@ -583,3 +664,74 @@ impl From<Bitmap> for arrow_buffer::buffer::NullBuffer {
         unsafe { arrow_buffer::buffer::NullBuffer::new_unchecked(buffer, null_count) }
     }
 }
+
+impl Splitable for Bitmap {
+    #[inline(always)]
+    fn check_bound(&self, offset: usize) -> bool {
+        offset <= self.len()
+    }
+
+    unsafe fn _split_at_unchecked(&self, offset: usize) -> (Self, Self) {
+        let bytes = &self.bytes;
+
+        if offset == 0 {
+            return (Self::new(), self.clone());
+        }
+        if offset == self.len() {
+            return (self.clone(), Self::new());
+        }
+
+        let ubcc = self.unset_bit_count_cache.load(Ordering::Relaxed);
+
+        let lhs_length = offset;
+        let rhs_length = self.length - offset;
+
+        let mut lhs_ubcc = UNKNOWN_BIT_COUNT;
+        let mut rhs_ubcc = UNKNOWN_BIT_COUNT;
+
+        if has_cached_unset_bit_count(ubcc) {
+            if ubcc == 0 {
+                lhs_ubcc = 0;
+                rhs_ubcc = 0;
+            } else if ubcc == self.length as u64 {
+                lhs_ubcc = offset as u64;
+                rhs_ubcc = (self.length - offset) as u64;
+            } else {
+                // If we keep all but a small portion of the array it is worth
+                // doing an eager re-count since we can reuse the old count via the
+                // inclusion-exclusion principle.
+                let small_portion = (self.length / 4).max(32);
+
+                if lhs_length <= rhs_length {
+                    if rhs_length + small_portion >= self.length {
+                        let count = count_zeros(&self.bytes, self.offset, lhs_length) as u64;
+                        lhs_ubcc = count;
+                        rhs_ubcc = ubcc - count;
+                    }
+                } else if lhs_length + small_portion >= self.length {
+                    let count = count_zeros(&self.bytes, self.offset + offset, rhs_length) as u64;
+                    lhs_ubcc = ubcc - count;
+                    rhs_ubcc = count;
+                }
+            }
+        }
+
+        debug_assert!(lhs_ubcc == UNKNOWN_BIT_COUNT || lhs_ubcc <= ubcc);
+        debug_assert!(rhs_ubcc == UNKNOWN_BIT_COUNT || rhs_ubcc <= ubcc);
+
+        (
+            Self {
+                bytes: bytes.clone(),
+                offset: self.offset,
+                length: lhs_length,
+                unset_bit_count_cache: AtomicU64::new(lhs_ubcc),
+            },
+            Self {
+                bytes: bytes.clone(),
+                offset: self.offset + offset,
+                length: rhs_length,
+                unset_bit_count_cache: AtomicU64::new(rhs_ubcc),
+            },
+        )
+    }
+}
