@@ -5,12 +5,10 @@ use crate::prelude::*;
 use crate::utils::_split_offsets;
 use crate::POOL;
 
-pub type ArrowChunk = RecordBatch<ArrayRef>;
-
-impl std::convert::TryFrom<(ArrowChunk, &[ArrowField])> for DataFrame {
+impl TryFrom<(RecordBatch, &[ArrowField])> for DataFrame {
     type Error = PolarsError;
 
-    fn try_from(arg: (ArrowChunk, &[ArrowField])) -> PolarsResult<DataFrame> {
+    fn try_from(arg: (RecordBatch, &[ArrowField])) -> PolarsResult<DataFrame> {
         let columns: PolarsResult<Vec<Series>> = arg
             .0
             .columns()
@@ -24,20 +22,14 @@ impl std::convert::TryFrom<(ArrowChunk, &[ArrowField])> for DataFrame {
 }
 
 impl DataFrame {
-    pub fn split_chunks(mut self) -> impl Iterator<Item = DataFrame> {
+    pub fn split_chunks(&mut self) -> impl Iterator<Item = DataFrame> + '_ {
         self.align_chunks();
 
         (0..self.n_chunks()).map(move |i| unsafe {
             let columns = self
                 .get_columns()
                 .iter()
-                .map(|s| {
-                    Series::from_chunks_and_dtype_unchecked(
-                        s.name(),
-                        vec![s.chunks()[i].clone()],
-                        s.dtype(),
-                    )
-                })
+                .map(|s| s.select_chunk(i))
                 .collect::<Vec<_>>();
 
             DataFrame::new_no_checks(columns)
