@@ -2,6 +2,7 @@
 #[cfg(feature = "python")]
 mod python;
 
+mod cached_arenas;
 mod err;
 #[cfg(not(target_arch = "wasm32"))]
 mod exitable;
@@ -14,8 +15,8 @@ pub mod pivot;
     feature = "csv",
     feature = "json"
 ))]
-use std::path::PathBuf;
-use std::sync::Arc;
+use std::path::Path;
+use std::sync::{Arc, Mutex};
 
 pub use anonymous_scan::*;
 #[cfg(feature = "csv")]
@@ -30,14 +31,15 @@ pub use ndjson::*;
 #[cfg(feature = "parquet")]
 pub use parquet::*;
 use polars_core::prelude::*;
+use polars_expr::{create_physical_expr, ExpressionConversionState};
 use polars_io::RowIndex;
+use polars_mem_engine::{create_physical_plan, Executor};
+use polars_ops::frame::JoinCoalesce;
 pub use polars_plan::frame::{AllowedOptimizations, OptState};
 use polars_plan::global::FETCH_ROWS;
 use smartstring::alias::String as SmartString;
 
-use crate::physical_plan::executors::Executor;
-use crate::physical_plan::planner::{create_physical_expr, create_physical_plan};
-use crate::physical_plan::state::ExecutionState;
+use crate::frame::cached_arenas::CachedArena;
 #[cfg(feature = "streaming")]
 use crate::physical_plan::streaming::insert_streaming_nodes;
 use crate::prelude::*;
@@ -53,6 +55,7 @@ impl IntoLazy for DataFrame {
         LazyFrame {
             logical_plan: lp,
             opt_state: Default::default(),
+            cached_arena: Default::default(),
         }
     }
 }
@@ -71,28 +74,30 @@ impl IntoLazy for LazyFrame {
 pub struct LazyFrame {
     pub logical_plan: DslPlan,
     pub(crate) opt_state: OptState,
+    pub(crate) cached_arena: Arc<Mutex<Option<CachedArena>>>,
 }
 
 impl From<DslPlan> for LazyFrame {
     fn from(plan: DslPlan) -> Self {
         Self {
             logical_plan: plan,
-            opt_state: OptState {
-                file_caching: true,
-                ..Default::default()
-            },
+            opt_state: OptState::default() | OptState::FILE_CACHING,
+            cached_arena: Default::default(),
         }
     }
 }
 
 impl LazyFrame {
-    /// Get a handle to the schema — a map from column names to data types — of the current
-    /// `LazyFrame` computation.
-    ///
-    /// Returns an `Err` if the logical plan has already encountered an error (i.e., if
-    /// `self.collect()` would fail), `Ok` otherwise.
-    pub fn schema(&self) -> PolarsResult<SchemaRef> {
-        self.logical_plan.compute_schema()
+    pub(crate) fn from_inner(
+        logical_plan: DslPlan,
+        opt_state: OptState,
+        cached_arena: Arc<Mutex<Option<CachedArena>>>,
+    ) -> Self {
+        Self {
+            logical_plan,
+            opt_state,
+            cached_arena,
+        }
     }
 
     pub(crate) fn get_plan_builder(self) -> DslBuilder {
@@ -107,6 +112,7 @@ impl LazyFrame {
         LazyFrame {
             logical_plan,
             opt_state,
+            cached_arena: Default::default(),
         }
     }
 
@@ -123,121 +129,116 @@ impl LazyFrame {
 
     /// Turn off all optimizations.
     pub fn without_optimizations(self) -> Self {
-        self.with_optimizations(OptState {
-            projection_pushdown: false,
-            predicate_pushdown: false,
-            type_coercion: true,
-            simplify_expr: false,
-            slice_pushdown: false,
-            // will be toggled by a scan operation such as csv scan or parquet scan
-            file_caching: false,
-            #[cfg(feature = "cse")]
-            comm_subplan_elim: false,
-            #[cfg(feature = "cse")]
-            comm_subexpr_elim: false,
-            streaming: false,
-            eager: false,
-            fast_projection: false,
-            row_estimate: false,
-        })
+        self.with_optimizations(OptState::from_bits_truncate(0) | OptState::TYPE_COERCION)
     }
 
     /// Toggle projection pushdown optimization.
     pub fn with_projection_pushdown(mut self, toggle: bool) -> Self {
-        self.opt_state.projection_pushdown = toggle;
+        self.opt_state.set(OptState::PROJECTION_PUSHDOWN, toggle);
+        self
+    }
+
+    /// Toggle cluster with columns optimization.
+    pub fn with_cluster_with_columns(mut self, toggle: bool) -> Self {
+        self.opt_state.set(OptState::CLUSTER_WITH_COLUMNS, toggle);
         self
     }
 
     /// Toggle predicate pushdown optimization.
     pub fn with_predicate_pushdown(mut self, toggle: bool) -> Self {
-        self.opt_state.predicate_pushdown = toggle;
+        self.opt_state.set(OptState::PREDICATE_PUSHDOWN, toggle);
         self
     }
 
     /// Toggle type coercion optimization.
     pub fn with_type_coercion(mut self, toggle: bool) -> Self {
-        self.opt_state.type_coercion = toggle;
+        self.opt_state.set(OptState::TYPE_COERCION, toggle);
         self
     }
 
     /// Toggle expression simplification optimization on or off.
     pub fn with_simplify_expr(mut self, toggle: bool) -> Self {
-        self.opt_state.simplify_expr = toggle;
+        self.opt_state.set(OptState::SIMPLIFY_EXPR, toggle);
         self
     }
 
     /// Toggle common subplan elimination optimization on or off
     #[cfg(feature = "cse")]
     pub fn with_comm_subplan_elim(mut self, toggle: bool) -> Self {
-        self.opt_state.comm_subplan_elim = toggle;
+        self.opt_state.set(OptState::COMM_SUBPLAN_ELIM, toggle);
         self
     }
 
     /// Toggle common subexpression elimination optimization on or off
     #[cfg(feature = "cse")]
     pub fn with_comm_subexpr_elim(mut self, toggle: bool) -> Self {
-        self.opt_state.comm_subexpr_elim = toggle;
+        self.opt_state.set(OptState::COMM_SUBEXPR_ELIM, toggle);
         self
     }
 
     /// Toggle slice pushdown optimization.
     pub fn with_slice_pushdown(mut self, toggle: bool) -> Self {
-        self.opt_state.slice_pushdown = toggle;
+        self.opt_state.set(OptState::SLICE_PUSHDOWN, toggle);
         self
     }
 
     /// Run nodes that are capably of doing so on the streaming engine.
     pub fn with_streaming(mut self, toggle: bool) -> Self {
-        self.opt_state.streaming = toggle;
+        self.opt_state.set(OptState::STREAMING, toggle);
+        self
+    }
+
+    pub fn with_new_streaming(mut self, toggle: bool) -> Self {
+        self.opt_state.set(OptState::NEW_STREAMING, toggle);
         self
     }
 
     /// Try to estimate the number of rows so that joins can determine which side to keep in memory.
     pub fn with_row_estimate(mut self, toggle: bool) -> Self {
-        self.opt_state.row_estimate = toggle;
+        self.opt_state.set(OptState::ROW_ESTIMATE, toggle);
         self
     }
 
     /// Run every node eagerly. This turns off multi-node optimizations.
     pub fn _with_eager(mut self, toggle: bool) -> Self {
-        self.opt_state.eager = toggle;
+        self.opt_state.set(OptState::EAGER, toggle);
         self
     }
 
     /// Return a String describing the naive (un-optimized) logical plan.
-    pub fn describe_plan(&self) -> String {
-        self.logical_plan.describe()
+    pub fn describe_plan(&self) -> PolarsResult<String> {
+        Ok(self.clone().to_alp()?.describe())
     }
 
     /// Return a String describing the naive (un-optimized) logical plan in tree format.
-    pub fn describe_plan_tree(&self) -> String {
-        self.logical_plan.describe_tree_format()
+    pub fn describe_plan_tree(&self) -> PolarsResult<String> {
+        Ok(self.clone().to_alp()?.describe_tree_format())
     }
 
-    fn optimized_plan(&self) -> PolarsResult<DslPlan> {
-        let mut expr_arena = Arena::with_capacity(64);
-        let mut lp_arena = Arena::with_capacity(64);
-        let lp_top = self.clone().optimize_with_scratch(
-            &mut lp_arena,
-            &mut expr_arena,
-            &mut vec![],
-            true,
-        )?;
-        Ok(node_to_lp(lp_top, &expr_arena, &mut lp_arena))
+    // @NOTE: this is used because we want to set the `enable_fmt` flag of `optimize_with_scratch`
+    // to `true` for describe.
+    fn _describe_to_alp_optimized(mut self) -> PolarsResult<IRPlan> {
+        let (mut lp_arena, mut expr_arena) = self.get_arenas();
+        let node = self.optimize_with_scratch(&mut lp_arena, &mut expr_arena, &mut vec![], true)?;
+
+        Ok(IRPlan::new(node, lp_arena, expr_arena))
     }
 
     /// Return a String describing the optimized logical plan.
     ///
     /// Returns `Err` if optimizing the logical plan fails.
     pub fn describe_optimized_plan(&self) -> PolarsResult<String> {
-        Ok(self.optimized_plan()?.describe())
+        Ok(self.clone()._describe_to_alp_optimized()?.describe())
     }
 
     /// Return a String describing the optimized logical plan in tree format.
     ///
     /// Returns `Err` if optimizing the logical plan fails.
     pub fn describe_optimized_plan_tree(&self) -> PolarsResult<String> {
-        Ok(self.optimized_plan()?.describe_tree_format())
+        Ok(self
+            .clone()
+            ._describe_to_alp_optimized()?
+            .describe_tree_format())
     }
 
     /// Return a String describing the logical plan.
@@ -248,7 +249,7 @@ impl LazyFrame {
         if optimized {
             self.describe_optimized_plan()
         } else {
-            Ok(self.describe_plan())
+            self.describe_plan()
         }
     }
 
@@ -286,7 +287,7 @@ impl LazyFrame {
     ///     df.lazy().sort(
     ///         &["sepal_width", "sepal_length"],
     ///         SortMultipleOptions::new()
-    ///             .with_order_descendings([false, true])
+    ///             .with_order_descending_multi([false, true])
     ///     )
     /// }
     /// ```
@@ -344,8 +345,11 @@ impl LazyFrame {
         sort_options: SortMultipleOptions,
     ) -> Self {
         // this will optimize to top-k
-        self.sort_by_exprs(by_exprs, sort_options.with_order_reversed())
-            .slice(0, k)
+        self.sort_by_exprs(
+            by_exprs,
+            sort_options.with_order_reversed().with_nulls_last(true),
+        )
+        .slice(0, k)
     }
 
     pub fn bottom_k<E: AsRef<[Expr]>>(
@@ -355,7 +359,8 @@ impl LazyFrame {
         sort_options: SortMultipleOptions,
     ) -> Self {
         // this will optimize to bottom-k
-        self.sort_by_exprs(by_exprs, sort_options).slice(0, k)
+        self.sort_by_exprs(by_exprs, sort_options.with_nulls_last(true))
+            .slice(0, k)
     }
 
     /// Reverse the `DataFrame` from top to bottom.
@@ -415,7 +420,10 @@ impl LazyFrame {
     /// Removes columns from the DataFrame.
     /// Note that it's better to only select the columns you need
     /// and let the projection pushdown optimize away the unneeded columns.
-    pub fn drop<I, T>(self, columns: I) -> Self
+    ///
+    /// If `strict` is `true`, then any given columns that are not in the schema will
+    /// give a [`PolarsError::ColumnNotFound`] error while materializing the [`LazyFrame`].
+    fn _drop<I, T>(self, columns: I, strict: bool) -> Self
     where
         I: IntoIterator<Item = T>,
         T: AsRef<str>,
@@ -426,10 +434,39 @@ impl LazyFrame {
             .collect::<PlHashSet<_>>();
 
         let opt_state = self.get_opt_state();
-        let lp = self.get_plan_builder().drop(to_drop).build();
+        let lp = self.get_plan_builder().drop(to_drop, strict).build();
         Self::from_logical_plan(lp, opt_state)
     }
 
+    /// Removes columns from the DataFrame.
+    /// Note that it's better to only select the columns you need
+    /// and let the projection pushdown optimize away the unneeded columns.
+    ///
+    /// Any given columns that are not in the schema will give a [`PolarsError::ColumnNotFound`]
+    /// error while materializing the [`LazyFrame`].
+    #[inline]
+    pub fn drop<I, T>(self, columns: I) -> Self
+    where
+        I: IntoIterator<Item = T>,
+        T: AsRef<str>,
+    {
+        self._drop(columns, true)
+    }
+
+    /// Removes columns from the DataFrame.
+    /// Note that it's better to only select the columns you need
+    /// and let the projection pushdown optimize away the unneeded columns.
+    ///
+    /// If a column name does not exist in the schema, it will quietly be ignored.
+    #[inline]
+    pub fn drop_no_validate<I, T>(self, columns: I) -> Self
+    where
+        I: IntoIterator<Item = T>,
+        T: AsRef<str>,
+    {
+        self._drop(columns, false)
+    }
+
     /// Shift the values by a given period and fill the parts that will be empty due to this operation
     /// with `Nones`.
     ///
@@ -519,16 +556,25 @@ impl LazyFrame {
         self.optimize_with_scratch(lp_arena, expr_arena, &mut vec![], false)
     }
 
-    pub fn to_alp_optimized(self) -> PolarsResult<(Node, Arena<IR>, Arena<AExpr>)> {
-        let mut lp_arena = Arena::with_capacity(16);
-        let mut expr_arena = Arena::with_capacity(16);
+    pub fn to_alp_optimized(mut self) -> PolarsResult<IRPlan> {
+        let (mut lp_arena, mut expr_arena) = self.get_arenas();
         let node =
             self.optimize_with_scratch(&mut lp_arena, &mut expr_arena, &mut vec![], false)?;
-        Ok((node, lp_arena, expr_arena))
+
+        Ok(IRPlan::new(node, lp_arena, expr_arena))
     }
 
-    pub fn to_alp(self) -> PolarsResult<(Node, Arena<IR>, Arena<AExpr>)> {
-        self.logical_plan.to_alp()
+    pub fn to_alp(mut self) -> PolarsResult<IRPlan> {
+        let (mut lp_arena, mut expr_arena) = self.get_arenas();
+        let node = to_alp(
+            self.logical_plan,
+            &mut expr_arena,
+            &mut lp_arena,
+            true,
+            true,
+        )?;
+        let plan = IRPlan::new(node, lp_arena, expr_arena);
+        Ok(plan)
     }
 
     pub(crate) fn optimize_with_scratch(
@@ -536,17 +582,14 @@ impl LazyFrame {
         lp_arena: &mut Arena<IR>,
         expr_arena: &mut Arena<AExpr>,
         scratch: &mut Vec<Node>,
-        _fmt: bool,
+        enable_fmt: bool,
     ) -> PolarsResult<Node> {
         #[allow(unused_mut)]
         let mut opt_state = self.opt_state;
-        let streaming = self.opt_state.streaming;
+        let streaming = self.opt_state.contains(OptState::STREAMING);
         #[cfg(feature = "cse")]
-        if streaming && self.opt_state.comm_subplan_elim {
-            polars_warn!(
-                "Cannot combine 'streaming' with 'comm_subplan_elim'. CSE will be turned off."
-            );
-            opt_state.comm_subplan_elim = false;
+        if streaming && self.opt_state.contains(OptState::COMM_SUBPLAN_ELIM) {
+            opt_state &= !OptState::COMM_SUBPLAN_ELIM;
         }
         let lp_top = optimize(
             self.logical_plan,
@@ -560,7 +603,7 @@ impl LazyFrame {
                     Context::Default,
                     expr_arena,
                     None,
-                    &mut Default::default(),
+                    &mut ExpressionConversionState::new(true, 0),
                 )
                 .ok()?;
                 let io_expr = phys_expr_to_io_expr(phys_expr);
@@ -576,42 +619,66 @@ impl LazyFrame {
                     lp_arena,
                     expr_arena,
                     scratch,
-                    _fmt,
+                    enable_fmt,
                     true,
-                    opt_state.row_estimate,
+                    opt_state.contains(OptState::ROW_ESTIMATE),
                 )?;
             }
             #[cfg(not(feature = "streaming"))]
             {
+                _ = enable_fmt;
                 panic!("activate feature 'streaming'")
             }
         }
+
         Ok(lp_top)
     }
 
-    #[allow(unused_mut)]
-    fn prepare_collect(
+    fn prepare_collect_post_opt<P>(
         mut self,
         check_sink: bool,
-    ) -> PolarsResult<(ExecutionState, Box<dyn Executor>, bool)> {
-        let mut expr_arena = Arena::with_capacity(256);
-        let mut lp_arena = Arena::with_capacity(128);
+        post_opt: P,
+    ) -> PolarsResult<(ExecutionState, Box<dyn Executor>, bool)>
+    where
+        P: Fn(Node, &mut Arena<IR>, &mut Arena<AExpr>) -> PolarsResult<()>,
+    {
+        let (mut lp_arena, mut expr_arena) = self.get_arenas();
+
         let mut scratch = vec![];
         let lp_top =
             self.optimize_with_scratch(&mut lp_arena, &mut expr_arena, &mut scratch, false)?;
 
+        post_opt(lp_top, &mut lp_arena, &mut expr_arena)?;
+
         // sink should be replaced
         let no_file_sink = if check_sink {
             !matches!(lp_arena.get(lp_top), IR::Sink { .. })
         } else {
             true
         };
-        let physical_plan = create_physical_plan(lp_top, &mut lp_arena, &mut expr_arena)?;
+        let physical_plan = create_physical_plan(lp_top, &mut lp_arena, &expr_arena)?;
 
         let state = ExecutionState::new();
         Ok((state, physical_plan, no_file_sink))
     }
 
+    // post_opt: A function that is called after optimization. This can be used to modify the IR jit.
+    pub fn _collect_post_opt<P>(self, post_opt: P) -> PolarsResult<DataFrame>
+    where
+        P: Fn(Node, &mut Arena<IR>, &mut Arena<AExpr>) -> PolarsResult<()>,
+    {
+        let (mut state, mut physical_plan, _) = self.prepare_collect_post_opt(false, post_opt)?;
+        physical_plan.execute(&mut state)
+    }
+
+    #[allow(unused_mut)]
+    fn prepare_collect(
+        self,
+        check_sink: bool,
+    ) -> PolarsResult<(ExecutionState, Box<dyn Executor>, bool)> {
+        self.prepare_collect_post_opt(check_sink, |_, _, _| Ok(()))
+    }
+
     /// Execute all the lazy operations and collect them into a [`DataFrame`].
     ///
     /// The query is optimized prior to execution.
@@ -630,8 +697,60 @@ impl LazyFrame {
     /// }
     /// ```
     pub fn collect(self) -> PolarsResult<DataFrame> {
-        let (mut state, mut physical_plan, _) = self.prepare_collect(false)?;
-        physical_plan.execute(&mut state)
+        #[cfg(feature = "new_streaming")]
+        {
+            let force_new_streaming = self.opt_state.contains(OptState::NEW_STREAMING);
+            let mut alp_plan = self.to_alp_optimized()?;
+            let stream_lp_top = alp_plan.lp_arena.add(IR::Sink {
+                input: alp_plan.lp_top,
+                payload: SinkType::Memory,
+            });
+
+            if force_new_streaming {
+                return polars_stream::run_query(
+                    stream_lp_top,
+                    alp_plan.lp_arena,
+                    &alp_plan.expr_arena,
+                );
+            }
+
+            if std::env::var("POLARS_AUTO_NEW_STREAMING")
+                .as_deref()
+                .unwrap_or("")
+                == "1"
+            {
+                let f = || {
+                    polars_stream::run_query(
+                        stream_lp_top,
+                        alp_plan.lp_arena.clone(),
+                        &alp_plan.expr_arena,
+                    )
+                };
+                match std::panic::catch_unwind(std::panic::AssertUnwindSafe(f)) {
+                    Ok(r) => return r,
+                    Err(e) => {
+                        // Fallback to normal engine if error is due to not being implemented,
+                        // otherwise propagate error.
+                        if e.downcast_ref::<&str>() != Some(&"not yet implemented") {
+                            if polars_core::config::verbose() {
+                                eprintln!("caught unimplemented error in new streaming engine, falling back to normal engine");
+                            }
+                            std::panic::resume_unwind(e);
+                        }
+                    },
+                }
+            }
+
+            let mut physical_plan = create_physical_plan(
+                alp_plan.lp_top,
+                &mut alp_plan.lp_arena,
+                &alp_plan.expr_arena,
+            )?;
+            let mut state = ExecutionState::new();
+            physical_plan.execute(&mut state)
+        }
+        #[cfg(not(feature = "new_streaming"))]
+        self._collect_post_opt(|_, _, _| Ok(()))
     }
 
     /// Profile a LazyFrame.
@@ -653,10 +772,14 @@ impl LazyFrame {
     /// into memory. This methods will return an error if the query cannot be completely done in a
     /// streaming fashion.
     #[cfg(feature = "parquet")]
-    pub fn sink_parquet(self, path: PathBuf, options: ParquetWriteOptions) -> PolarsResult<()> {
+    pub fn sink_parquet(
+        self,
+        path: impl AsRef<Path>,
+        options: ParquetWriteOptions,
+    ) -> PolarsResult<()> {
         self.sink(
             SinkType::File {
-                path: Arc::new(path),
+                path: Arc::new(path.as_ref().to_path_buf()),
                 file_type: FileType::Parquet(options),
             },
             "collect().write_parquet()",
@@ -688,10 +811,10 @@ impl LazyFrame {
     /// into memory. This methods will return an error if the query cannot be completely done in a
     /// streaming fashion.
     #[cfg(feature = "ipc")]
-    pub fn sink_ipc(self, path: PathBuf, options: IpcWriterOptions) -> PolarsResult<()> {
+    pub fn sink_ipc(self, path: impl AsRef<Path>, options: IpcWriterOptions) -> PolarsResult<()> {
         self.sink(
             SinkType::File {
-                path: Arc::new(path),
+                path: Arc::new(path.as_ref().to_path_buf()),
                 file_type: FileType::Ipc(options),
             },
             "collect().write_ipc()",
@@ -710,7 +833,7 @@ impl LazyFrame {
         cloud_options: Option<polars_io::cloud::CloudOptions>,
         ipc_options: IpcWriterOptions,
     ) -> PolarsResult<()> {
-        self.opt_state.streaming = true;
+        self.opt_state |= OptState::STREAMING;
         self.logical_plan = DslPlan::Sink {
             input: Arc::new(self.logical_plan),
             payload: SinkType::Cloud {
@@ -733,10 +856,10 @@ impl LazyFrame {
     /// into memory. This methods will return an error if the query cannot be completely done in a
     /// streaming fashion.
     #[cfg(feature = "csv")]
-    pub fn sink_csv(self, path: PathBuf, options: CsvWriterOptions) -> PolarsResult<()> {
+    pub fn sink_csv(self, path: impl AsRef<Path>, options: CsvWriterOptions) -> PolarsResult<()> {
         self.sink(
             SinkType::File {
-                path: Arc::new(path),
+                path: Arc::new(path.as_ref().to_path_buf()),
                 file_type: FileType::Csv(options),
             },
             "collect().write_csv()",
@@ -747,10 +870,10 @@ impl LazyFrame {
     /// into memory. This methods will return an error if the query cannot be completely done in a
     /// streaming fashion.
     #[cfg(feature = "json")]
-    pub fn sink_json(self, path: PathBuf, options: JsonWriterOptions) -> PolarsResult<()> {
+    pub fn sink_json(self, path: impl AsRef<Path>, options: JsonWriterOptions) -> PolarsResult<()> {
         self.sink(
             SinkType::File {
-                path: Arc::new(path),
+                path: Arc::new(path.as_ref().to_path_buf()),
                 file_type: FileType::Json(options),
             },
             "collect().write_ndjson()` or `collect().write_json()",
@@ -765,7 +888,7 @@ impl LazyFrame {
         feature = "json",
     ))]
     fn sink(mut self, payload: SinkType, msg_alternative: &str) -> Result<(), PolarsError> {
-        self.opt_state.streaming = true;
+        self.opt_state |= OptState::STREAMING;
         self.logical_plan = DslPlan::Sink {
             input: Arc::new(self.logical_plan),
             payload,
@@ -834,6 +957,7 @@ impl LazyFrame {
             ProjectionOptions {
                 run_parallel: true,
                 duplicate_check: true,
+                should_broadcast: true,
             },
         )
     }
@@ -845,6 +969,7 @@ impl LazyFrame {
             ProjectionOptions {
                 run_parallel: false,
                 duplicate_check: true,
+                should_broadcast: true,
             },
         )
     }
@@ -916,7 +1041,7 @@ impl LazyFrame {
     /// *group_by_dynamic*
     #[cfg(feature = "dynamic_group_by")]
     pub fn rolling<E: AsRef<[Expr]>>(
-        self,
+        mut self,
         index_column: Expr,
         group_by: E,
         mut options: RollingGroupOptions,
@@ -961,7 +1086,7 @@ impl LazyFrame {
     /// with a ordinary group_by on these keys.
     #[cfg(feature = "dynamic_group_by")]
     pub fn group_by_dynamic<E: AsRef<[Expr]>>(
-        self,
+        mut self,
         index_column: Expr,
         group_by: E,
         mut options: DynamicGroupOptions,
@@ -1049,11 +1174,16 @@ impl LazyFrame {
 
     /// Creates the Cartesian product from both frames, preserving the order of the left keys.
     #[cfg(feature = "cross_join")]
-    pub fn cross_join(self, other: LazyFrame) -> LazyFrame {
-        self.join(other, vec![], vec![], JoinArgs::new(JoinType::Cross))
+    pub fn cross_join(self, other: LazyFrame, suffix: Option<String>) -> LazyFrame {
+        self.join(
+            other,
+            vec![],
+            vec![],
+            JoinArgs::new(JoinType::Cross).with_suffix(suffix),
+        )
     }
 
-    /// Left join this query with another lazy query.
+    /// Left outer join this query with another lazy query.
     ///
     /// Matches on the values of the expressions `left_on` and `right_on`. For more
     /// flexible join logic, see [`join`](LazyFrame::join) or
@@ -1103,7 +1233,7 @@ impl LazyFrame {
         )
     }
 
-    /// Outer join this query with another lazy query.
+    /// Full outer join this query with another lazy query.
     ///
     /// Matches on the values of the expressions `left_on` and `right_on`. For more
     /// flexible join logic, see [`join`](LazyFrame::join) or
@@ -1114,17 +1244,17 @@ impl LazyFrame {
     /// ```rust
     /// use polars_core::prelude::*;
     /// use polars_lazy::prelude::*;
-    /// fn outer_join_dataframes(ldf: LazyFrame, other: LazyFrame) -> LazyFrame {
+    /// fn full_join_dataframes(ldf: LazyFrame, other: LazyFrame) -> LazyFrame {
     ///         ldf
-    ///         .outer_join(other, col("foo"), col("bar"))
+    ///         .full_join(other, col("foo"), col("bar"))
     /// }
     /// ```
-    pub fn outer_join<E: Into<Expr>>(self, other: LazyFrame, left_on: E, right_on: E) -> LazyFrame {
+    pub fn full_join<E: Into<Expr>>(self, other: LazyFrame, left_on: E, right_on: E) -> LazyFrame {
         self.join(
             other,
             [left_on.into()],
             [right_on.into()],
-            JoinArgs::new(JoinType::Outer { coalesce: false }),
+            JoinArgs::new(JoinType::Full),
         )
     }
 
@@ -1183,7 +1313,9 @@ impl LazyFrame {
         args: JoinArgs,
     ) -> LazyFrame {
         // if any of the nodes reads from files we must activate this this plan as well.
-        self.opt_state.file_caching |= other.opt_state.file_caching;
+        if other.opt_state.contains(OptState::FILE_CACHING) {
+            self.opt_state |= OptState::FILE_CACHING;
+        }
 
         let left_on = left_on.as_ref().to_vec();
         let right_on = right_on.as_ref().to_vec();
@@ -1195,14 +1327,13 @@ impl LazyFrame {
             .right_on(right_on)
             .how(args.how)
             .validate(args.validation)
+            .coalesce(args.coalesce)
             .join_nulls(args.join_nulls);
 
         if let Some(suffix) = args.suffix {
             builder = builder.suffix(suffix);
         }
-
         // Note: args.slice is set by the optimizer
-
         builder.finish()
     }
 
@@ -1241,6 +1372,7 @@ impl LazyFrame {
                 ProjectionOptions {
                     run_parallel: false,
                     duplicate_check: true,
+                    should_broadcast: true,
                 },
             )
             .build();
@@ -1268,6 +1400,7 @@ impl LazyFrame {
             ProjectionOptions {
                 run_parallel: true,
                 duplicate_check: true,
+                should_broadcast: true,
             },
         )
     }
@@ -1280,6 +1413,7 @@ impl LazyFrame {
             ProjectionOptions {
                 run_parallel: false,
                 duplicate_check: true,
+                should_broadcast: true,
             },
         )
     }
@@ -1320,9 +1454,9 @@ impl LazyFrame {
     /// Aggregated columns will have the same names as the original columns.
     ///
     /// - Boolean columns will sum to a `u32` containing the number of `true`s.
-    /// - For integer columns, the ordinary checks for overflow are performed: if running
-    /// in `debug` mode, overflows will panic, whereas in `release` mode overflows will
-    /// silently wrap.
+    /// - For integer columns, the ordinary checks for overflow are performed:
+    ///   if running in `debug` mode, overflows will panic, whereas in `release` mode overflows will
+    ///   silently wrap.
     /// - String columns will sum to None.
     pub fn sum(self) -> Self {
         self.map_private(DslFunction::Stats(StatsFunction::Sum))
@@ -1489,12 +1623,12 @@ impl LazyFrame {
         self.slice(neg_tail, n)
     }
 
-    /// Melt the DataFrame from wide to long format.
+    /// Unpivot the DataFrame from wide to long format.
     ///
-    /// See [`MeltArgs`] for information on how to melt a DataFrame.
-    pub fn melt(self, args: MeltArgs) -> LazyFrame {
+    /// See [`UnpivotArgs`] for information on how to unpivot a DataFrame.
+    pub fn unpivot(self, args: UnpivotArgs) -> LazyFrame {
         let opt_state = self.get_opt_state();
-        let lp = self.get_plan_builder().melt(args).build();
+        let lp = self.get_plan_builder().unpivot(args).build();
         Self::from_logical_plan(lp, opt_state)
     }
 
@@ -1579,7 +1713,7 @@ impl LazyFrame {
                 ..
             } if !matches!(scan_type, FileScan::Anonymous { .. }) => {
                 options.row_index = Some(RowIndex {
-                    name: name.to_string(),
+                    name: Arc::from(name),
                     offset: offset.unwrap_or(0),
                 });
                 false
@@ -1652,6 +1786,7 @@ impl From<LazyGroupBy> for LazyFrame {
         Self {
             logical_plan: lgb.logical_plan,
             opt_state: lgb.opt_state,
+            cached_arena: Default::default(),
         }
     }
 }
@@ -1764,6 +1899,7 @@ pub struct JoinBuilder {
     force_parallel: bool,
     suffix: Option<String>,
     validation: JoinValidation,
+    coalesce: JoinCoalesce,
     join_nulls: bool,
 }
 impl JoinBuilder {
@@ -1780,6 +1916,7 @@ impl JoinBuilder {
             join_nulls: false,
             suffix: None,
             validation: Default::default(),
+            coalesce: Default::default(),
         }
     }
 
@@ -1851,13 +1988,21 @@ impl JoinBuilder {
         self
     }
 
+    /// Whether to coalesce join columns.
+    pub fn coalesce(mut self, coalesce: JoinCoalesce) -> Self {
+        self.coalesce = coalesce;
+        self
+    }
+
     /// Finish builder
     pub fn finish(self) -> LazyFrame {
         let mut opt_state = self.lf.opt_state;
         let other = self.other.expect("with not set");
 
-        // if any of the nodes reads from files we must activate this this plan as well.
-        opt_state.file_caching |= other.opt_state.file_caching;
+        // If any of the nodes reads from files we must activate this this plan as well.
+        if other.opt_state.contains(OptState::FILE_CACHING) {
+            opt_state |= OptState::FILE_CACHING;
+        }
 
         let args = JoinArgs {
             how: self.how,
@@ -1865,6 +2010,7 @@ impl JoinBuilder {
             suffix: self.suffix,
             slice: None,
             join_nulls: self.join_nulls,
+            coalesce: self.coalesce,
         };
 
         let lp = self
