@@ -1,4 +1,6 @@
 use arrow::array::PrimitiveArray;
+use polars_core::series::BitRepr;
+use polars_core::utils::split;
 use polars_core::with_match_physical_float_polars_type;
 use polars_utils::hashing::DirtyHash;
 use polars_utils::nulls::IsNull;
@@ -19,12 +21,14 @@ pub trait SeriesJoin: SeriesSealed + Sized {
         let (lhs, rhs) = (s_self.to_physical_repr(), other.to_physical_repr());
         validate.validate_probe(&lhs, &rhs, false)?;
 
-        use DataType::*;
+        let lhs_dtype = lhs.dtype();
+        let rhs_dtype = rhs.dtype();
 
-        match lhs.dtype() {
-            String | Binary => {
-                let lhs = lhs.cast(&Binary).unwrap();
-                let rhs = rhs.cast(&Binary).unwrap();
+        use DataType as T;
+        match lhs_dtype {
+            T::String | T::Binary => {
+                let lhs = lhs.cast(&T::Binary).unwrap();
+                let rhs = rhs.cast(&T::Binary).unwrap();
                 let lhs = lhs.binary().unwrap();
                 let rhs = rhs.binary().unwrap();
                 let (lhs, rhs, _, _) = prepare_binary::<BinaryType>(lhs, rhs, false);
@@ -32,7 +36,7 @@ pub trait SeriesJoin: SeriesSealed + Sized {
                 let rhs = rhs.iter().map(|v| v.as_slice()).collect::<Vec<_>>();
                 hash_join_tuples_left(lhs, rhs, None, None, validate, join_nulls)
             },
-            BinaryOffset => {
+            T::BinaryOffset => {
                 let lhs = lhs.binary_offset().unwrap();
                 let rhs = rhs.binary_offset().unwrap();
                 let (lhs, rhs, _, _) = prepare_binary::<BinaryOffsetType>(lhs, rhs, false);
@@ -41,37 +45,59 @@ pub trait SeriesJoin: SeriesSealed + Sized {
                 let rhs = rhs.iter().map(|k| k.as_slice()).collect::<Vec<_>>();
                 hash_join_tuples_left(lhs, rhs, None, None, validate, join_nulls)
             },
+            x if x.is_float() => {
+                with_match_physical_float_polars_type!(lhs.dtype(), |$T| {
+                    let lhs: &ChunkedArray<$T> = lhs.as_ref().as_ref().as_ref();
+                    let rhs: &ChunkedArray<$T> = rhs.as_ref().as_ref().as_ref();
+                    num_group_join_left(lhs, rhs, validate, join_nulls)
+                })
+            },
             _ => {
-                if lhs.dtype().is_float() {
-                    with_match_physical_float_polars_type!(lhs.dtype(), |$T| {
-                        let lhs: &ChunkedArray<$T> = lhs.as_ref().as_ref().as_ref();
-                        let rhs: &ChunkedArray<$T> = rhs.as_ref().as_ref().as_ref();
-                        num_group_join_left(lhs, rhs, validate, join_nulls)
-                    })
-                } else if s_self.bit_repr_is_large() {
-                    let lhs = lhs.bit_repr_large();
-                    let rhs = rhs.bit_repr_large();
-                    num_group_join_left(&lhs, &rhs, validate, join_nulls)
-                } else {
-                    let lhs = lhs.bit_repr_small();
-                    let rhs = rhs.bit_repr_small();
-                    num_group_join_left(&lhs, &rhs, validate, join_nulls)
+                let lhs = s_self.bit_repr();
+                let rhs = other.bit_repr();
+
+                let (Some(lhs), Some(rhs)) = (lhs, rhs) else {
+                    polars_bail!(nyi = "Hash Left Join between {lhs_dtype} and {rhs_dtype}");
+                };
+
+                use BitRepr as B;
+                match (lhs, rhs) {
+                    (B::Small(lhs), B::Small(rhs)) => {
+                        // Turbofish: see #17137.
+                        num_group_join_left::<UInt32Type>(&lhs, &rhs, validate, join_nulls)
+                    },
+                    (B::Large(lhs), B::Large(rhs)) => {
+                        // Turbofish: see #17137.
+                        num_group_join_left::<UInt64Type>(&lhs, &rhs, validate, join_nulls)
+                    },
+                    _ => {
+                        polars_bail!(
+                        nyi = "Mismatch bit repr Hash Left Join between {lhs_dtype} and {rhs_dtype}",
+                    );
+                    },
                 }
             },
         }
     }
 
     #[cfg(feature = "semi_anti_join")]
-    fn hash_join_semi_anti(&self, other: &Series, anti: bool, join_nulls: bool) -> Vec<IdxSize> {
+    fn hash_join_semi_anti(
+        &self,
+        other: &Series,
+        anti: bool,
+        join_nulls: bool,
+    ) -> PolarsResult<Vec<IdxSize>> {
         let s_self = self.as_series();
         let (lhs, rhs) = (s_self.to_physical_repr(), other.to_physical_repr());
 
-        use DataType::*;
+        let lhs_dtype = lhs.dtype();
+        let rhs_dtype = rhs.dtype();
 
-        match lhs.dtype() {
-            String | Binary => {
-                let lhs = lhs.cast(&Binary).unwrap();
-                let rhs = rhs.cast(&Binary).unwrap();
+        use DataType as T;
+        Ok(match lhs_dtype {
+            T::String | T::Binary => {
+                let lhs = lhs.cast(&T::Binary).unwrap();
+                let rhs = rhs.cast(&T::Binary).unwrap();
                 let lhs = lhs.binary().unwrap();
                 let rhs = rhs.binary().unwrap();
                 let (lhs, rhs, _, _) = prepare_binary::<BinaryType>(lhs, rhs, false);
@@ -84,7 +110,7 @@ pub trait SeriesJoin: SeriesSealed + Sized {
                     hash_join_tuples_left_semi(lhs, rhs, join_nulls)
                 }
             },
-            BinaryOffset => {
+            T::BinaryOffset => {
                 let lhs = lhs.binary_offset().unwrap();
                 let rhs = rhs.binary_offset().unwrap();
                 let (lhs, rhs, _, _) = prepare_binary::<BinaryOffsetType>(lhs, rhs, false);
@@ -97,24 +123,39 @@ pub trait SeriesJoin: SeriesSealed + Sized {
                     hash_join_tuples_left_semi(lhs, rhs, join_nulls)
                 }
             },
+            x if x.is_float() => {
+                with_match_physical_float_polars_type!(lhs.dtype(), |$T| {
+                    let lhs: &ChunkedArray<$T> = lhs.as_ref().as_ref().as_ref();
+                    let rhs: &ChunkedArray<$T> = rhs.as_ref().as_ref().as_ref();
+                    num_group_join_anti_semi(lhs, rhs, anti, join_nulls)
+                })
+            },
             _ => {
-                if lhs.dtype().is_float() {
-                    with_match_physical_float_polars_type!(lhs.dtype(), |$T| {
-                        let lhs: &ChunkedArray<$T> = lhs.as_ref().as_ref().as_ref();
-                        let rhs: &ChunkedArray<$T> = rhs.as_ref().as_ref().as_ref();
-                        num_group_join_anti_semi(lhs, rhs, anti, join_nulls)
-                    })
-                } else if s_self.bit_repr_is_large() {
-                    let lhs = lhs.bit_repr_large();
-                    let rhs = rhs.bit_repr_large();
-                    num_group_join_anti_semi(&lhs, &rhs, anti, join_nulls)
-                } else {
-                    let lhs = lhs.bit_repr_small();
-                    let rhs = rhs.bit_repr_small();
-                    num_group_join_anti_semi(&lhs, &rhs, anti, join_nulls)
+                let lhs = s_self.bit_repr();
+                let rhs = other.bit_repr();
+
+                let (Some(lhs), Some(rhs)) = (lhs, rhs) else {
+                    polars_bail!(nyi = "Hash Semi-Anti Join between {lhs_dtype} and {rhs_dtype}");
+                };
+
+                use BitRepr as B;
+                match (lhs, rhs) {
+                    (B::Small(lhs), B::Small(rhs)) => {
+                        // Turbofish: see #17137.
+                        num_group_join_anti_semi::<UInt32Type>(&lhs, &rhs, anti, join_nulls)
+                    },
+                    (B::Large(lhs), B::Large(rhs)) => {
+                        // Turbofish: see #17137.
+                        num_group_join_anti_semi::<UInt64Type>(&lhs, &rhs, anti, join_nulls)
+                    },
+                    _ => {
+                        polars_bail!(
+                            nyi = "Mismatch bit repr Hash Semi-Anti Join between {lhs_dtype} and {rhs_dtype}",
+                        );
+                    },
                 }
             },
-        }
+        })
     }
 
     // returns the join tuples and whether or not the lhs tuples are sorted
@@ -128,11 +169,14 @@ pub trait SeriesJoin: SeriesSealed + Sized {
         let (lhs, rhs) = (s_self.to_physical_repr(), other.to_physical_repr());
         validate.validate_probe(&lhs, &rhs, true)?;
 
-        use DataType::*;
-        match lhs.dtype() {
-            String | Binary => {
-                let lhs = lhs.cast(&Binary).unwrap();
-                let rhs = rhs.cast(&Binary).unwrap();
+        let lhs_dtype = lhs.dtype();
+        let rhs_dtype = rhs.dtype();
+
+        use DataType as T;
+        match lhs_dtype {
+            T::String | T::Binary => {
+                let lhs = lhs.cast(&T::Binary).unwrap();
+                let rhs = rhs.cast(&T::Binary).unwrap();
                 let lhs = lhs.binary().unwrap();
                 let rhs = rhs.binary().unwrap();
                 let (lhs, rhs, swapped, _) = prepare_binary::<BinaryType>(lhs, rhs, true);
@@ -144,7 +188,7 @@ pub trait SeriesJoin: SeriesSealed + Sized {
                     !swapped,
                 ))
             },
-            BinaryOffset => {
+            T::BinaryOffset => {
                 let lhs = lhs.binary_offset().unwrap();
                 let rhs = rhs.binary_offset()?;
                 let (lhs, rhs, swapped, _) = prepare_binary::<BinaryOffsetType>(lhs, rhs, true);
@@ -156,21 +200,36 @@ pub trait SeriesJoin: SeriesSealed + Sized {
                     !swapped,
                 ))
             },
+            x if x.is_float() => {
+                with_match_physical_float_polars_type!(lhs.dtype(), |$T| {
+                    let lhs: &ChunkedArray<$T> = lhs.as_ref().as_ref().as_ref();
+                    let rhs: &ChunkedArray<$T> = rhs.as_ref().as_ref().as_ref();
+                    group_join_inner::<$T>(lhs, rhs, validate, join_nulls)
+                })
+            },
             _ => {
-                if lhs.dtype().is_float() {
-                    with_match_physical_float_polars_type!(lhs.dtype(), |$T| {
-                        let lhs: &ChunkedArray<$T> = lhs.as_ref().as_ref().as_ref();
-                        let rhs: &ChunkedArray<$T> = rhs.as_ref().as_ref().as_ref();
-                        group_join_inner::<$T>(lhs, rhs, validate, join_nulls)
-                    })
-                } else if s_self.bit_repr_is_large() {
-                    let lhs = s_self.bit_repr_large();
-                    let rhs = other.bit_repr_large();
-                    group_join_inner::<UInt64Type>(&lhs, &rhs, validate, join_nulls)
-                } else {
-                    let lhs = s_self.bit_repr_small();
-                    let rhs = other.bit_repr_small();
-                    group_join_inner::<UInt32Type>(&lhs, &rhs, validate, join_nulls)
+                let lhs = s_self.bit_repr();
+                let rhs = other.bit_repr();
+
+                let (Some(lhs), Some(rhs)) = (lhs, rhs) else {
+                    polars_bail!(nyi = "Hash Inner Join between {lhs_dtype} and {rhs_dtype}");
+                };
+
+                use BitRepr as B;
+                match (lhs, rhs) {
+                    (B::Small(lhs), B::Small(rhs)) => {
+                        // Turbofish: see #17137.
+                        group_join_inner::<UInt32Type>(&lhs, &rhs, validate, join_nulls)
+                    },
+                    (B::Large(lhs), BitRepr::Large(rhs)) => {
+                        // Turbofish: see #17137.
+                        group_join_inner::<UInt64Type>(&lhs, &rhs, validate, join_nulls)
+                    },
+                    _ => {
+                        polars_bail!(
+                            nyi = "Mismatch bit repr Hash Inner Join between {lhs_dtype} and {rhs_dtype}"
+                        );
+                    },
                 }
             },
         }
@@ -186,11 +245,14 @@ pub trait SeriesJoin: SeriesSealed + Sized {
         let (lhs, rhs) = (s_self.to_physical_repr(), other.to_physical_repr());
         validate.validate_probe(&lhs, &rhs, true)?;
 
-        use DataType::*;
-        match lhs.dtype() {
-            String | Binary => {
-                let lhs = lhs.cast(&Binary).unwrap();
-                let rhs = rhs.cast(&Binary).unwrap();
+        let lhs_dtype = lhs.dtype();
+        let rhs_dtype = rhs.dtype();
+
+        use DataType as T;
+        match lhs_dtype {
+            T::String | T::Binary => {
+                let lhs = lhs.cast(&T::Binary).unwrap();
+                let rhs = rhs.cast(&T::Binary).unwrap();
                 let lhs = lhs.binary().unwrap();
                 let rhs = rhs.binary().unwrap();
                 let (lhs, rhs, swapped, _) = prepare_binary::<BinaryType>(lhs, rhs, true);
@@ -199,7 +261,7 @@ pub trait SeriesJoin: SeriesSealed + Sized {
                 let rhs = rhs.iter().map(|k| k.as_slice()).collect::<Vec<_>>();
                 hash_join_tuples_outer(lhs, rhs, swapped, validate, join_nulls)
             },
-            BinaryOffset => {
+            T::BinaryOffset => {
                 let lhs = lhs.binary_offset().unwrap();
                 let rhs = rhs.binary_offset()?;
                 let (lhs, rhs, swapped, _) = prepare_binary::<BinaryOffsetType>(lhs, rhs, true);
@@ -208,21 +270,31 @@ pub trait SeriesJoin: SeriesSealed + Sized {
                 let rhs = rhs.iter().map(|k| k.as_slice()).collect::<Vec<_>>();
                 hash_join_tuples_outer(lhs, rhs, swapped, validate, join_nulls)
             },
+            x if x.is_float() => {
+                with_match_physical_float_polars_type!(lhs.dtype(), |$T| {
+                    let lhs: &ChunkedArray<$T> = lhs.as_ref().as_ref().as_ref();
+                    let rhs: &ChunkedArray<$T> = rhs.as_ref().as_ref().as_ref();
+                    hash_join_outer(lhs, rhs, validate, join_nulls)
+                })
+            },
             _ => {
-                if lhs.dtype().is_float() {
-                    with_match_physical_float_polars_type!(lhs.dtype(), |$T| {
-                        let lhs: &ChunkedArray<$T> = lhs.as_ref().as_ref().as_ref();
-                        let rhs: &ChunkedArray<$T> = rhs.as_ref().as_ref().as_ref();
-                        hash_join_outer(lhs, rhs, validate, join_nulls)
-                    })
-                } else if s_self.bit_repr_is_large() {
-                    let lhs = s_self.bit_repr_large();
-                    let rhs = other.bit_repr_large();
-                    hash_join_outer(&lhs, &rhs, validate, join_nulls)
-                } else {
-                    let lhs = s_self.bit_repr_small();
-                    let rhs = other.bit_repr_small();
-                    hash_join_outer(&lhs, &rhs, validate, join_nulls)
+                let (Some(lhs), Some(rhs)) = (s_self.bit_repr(), other.bit_repr()) else {
+                    polars_bail!(nyi = "Hash Join Outer between {lhs_dtype} and {rhs_dtype}");
+                };
+
+                use BitRepr as B;
+                match (lhs, rhs) {
+                    (B::Small(lhs), B::Small(rhs)) => {
+                        // Turbofish: see #17137.
+                        hash_join_outer::<UInt32Type>(&lhs, &rhs, validate, join_nulls)
+                    },
+                    (B::Large(lhs), B::Large(rhs)) => {
+                        // Turbofish: see #17137.
+                        hash_join_outer::<UInt64Type>(&lhs, &rhs, validate, join_nulls)
+                    },
+                    _ => {
+                        polars_bail!(nyi = "Mismatch bit repr Hash Join Outer between {lhs_dtype} and {rhs_dtype}");
+                    },
                 }
             },
         }
@@ -261,8 +333,8 @@ where
 {
     let n_threads = POOL.current_num_threads();
     let (a, b, swapped) = det_hash_prone_order!(left, right);
-    let splitted_a = split_ca(a, n_threads).unwrap();
-    let splitted_b = split_ca(b, n_threads).unwrap();
+    let splitted_a = split(a, n_threads);
+    let splitted_b = split(b, n_threads);
     let splitted_a = get_arrays(&splitted_a);
     let splitted_b = get_arrays(&splitted_b);
 
@@ -346,8 +418,8 @@ where
     <Option<T::Native> as ToTotalOrd>::TotalOrdItem: Send + Sync + DirtyHash,
 {
     let n_threads = POOL.current_num_threads();
-    let splitted_a = split_ca(left, n_threads).unwrap();
-    let splitted_b = split_ca(right, n_threads).unwrap();
+    let splitted_a = split(left, n_threads);
+    let splitted_b = split(right, n_threads);
     match (
         left.null_count(),
         right.null_count(),
@@ -405,8 +477,8 @@ where
     let (a, b, swapped) = det_hash_prone_order!(ca_in, other);
 
     let n_partitions = _set_partition_size();
-    let splitted_a = split_ca(a, n_partitions).unwrap();
-    let splitted_b = split_ca(b, n_partitions).unwrap();
+    let splitted_a = split(a, n_partitions);
+    let splitted_b = split(b, n_partitions);
 
     match (a.null_count(), b.null_count()) {
         (0, 0) => {
@@ -496,8 +568,8 @@ where
     <Option<T::Native> as ToTotalOrd>::TotalOrdItem: Send + Sync + DirtyHash + IsNull,
 {
     let n_threads = POOL.current_num_threads();
-    let splitted_a = split_ca(left, n_threads).unwrap();
-    let splitted_b = split_ca(right, n_threads).unwrap();
+    let splitted_a = split(left, n_threads);
+    let splitted_b = split(right, n_threads);
     match (
         left.null_count(),
         right.null_count(),
