@@ -1,10 +1,10 @@
 use arrow::array::{MutableArray, MutableBinaryViewArray, ViewType};
 use polars_error::PolarsResult;
 
-use crate::parquet::statistics::{BinaryStatistics, Statistics as ParquetStatistics};
+use crate::parquet::statistics::BinaryStatistics;
 
 pub(super) fn push<T: ViewType + ?Sized>(
-    from: Option<&dyn ParquetStatistics>,
+    from: Option<&BinaryStatistics>,
     min: &mut dyn MutableArray,
     max: &mut dyn MutableArray,
 ) -> PolarsResult<()> {
@@ -16,7 +16,7 @@ pub(super) fn push<T: ViewType + ?Sized>(
         .as_mut_any()
         .downcast_mut::<MutableBinaryViewArray<T>>()
         .unwrap();
-    let from = from.map(|s| s.as_any().downcast_ref::<BinaryStatistics>().unwrap());
+
     min.push(from.and_then(|s| {
         let opt_b = s.min_value.as_deref();
         unsafe { opt_b.map(|b| T::from_bytes_unchecked(b)) }
@@ -25,5 +25,6 @@ pub(super) fn push<T: ViewType + ?Sized>(
         let opt_b = s.max_value.as_deref();
         unsafe { opt_b.map(|b| T::from_bytes_unchecked(b)) }
     }));
+
     Ok(())
 }
