@@ -3,13 +3,14 @@
 #[cfg(feature = "dtype-categorical")]
 pub mod cat;
 
-#[cfg(feature = "rolling_window")]
+#[cfg(any(feature = "rolling_window", feature = "rolling_window_by"))]
 use std::any::Any;
 
 #[cfg(feature = "dtype-categorical")]
 pub use cat::*;
-#[cfg(feature = "rolling_window")]
+#[cfg(feature = "rolling_window_by")]
 pub(crate) use polars_time::prelude::*;
+
 mod arithmetic;
 mod arity;
 #[cfg(feature = "dtype-array")]
@@ -20,7 +21,7 @@ pub mod dt;
 mod expr;
 mod expr_dyn_fn;
 mod from;
-pub(crate) mod function_expr;
+pub mod function_expr;
 pub mod functions;
 mod list;
 #[cfg(feature = "meta")]
@@ -55,6 +56,8 @@ pub use list::*;
 pub use meta::*;
 pub use name::*;
 pub use options::*;
+use polars_core::chunked_array::cast::CastOptions;
+use polars_core::error::feature_gated;
 use polars_core::prelude::*;
 #[cfg(feature = "diff")]
 use polars_core::series::ops::NullBehavior;
@@ -66,7 +69,7 @@ pub use struct_::*;
 pub use udf::UserDefinedFunction;
 
 use crate::constants::MAP_LIST_NAME;
-pub use crate::logical_plan::lit;
+pub use crate::plans::lit;
 use crate::prelude::*;
 
 impl Expr {
@@ -183,7 +186,15 @@ impl Expr {
 
     /// Drop null values.
     pub fn drop_nulls(self) -> Self {
-        self.apply_private(FunctionExpr::DropNulls)
+        Expr::Function {
+            input: vec![self],
+            function: FunctionExpr::DropNulls,
+            options: FunctionOptions {
+                collect_groups: ApplyOptions::GroupWise,
+                flags: FunctionFlags::default() | FunctionFlags::ALLOW_EMPTY_INPUTS,
+                ..Default::default()
+            },
+        }
     }
 
     /// Drop NaN values.
@@ -301,7 +312,7 @@ impl Expr {
     pub fn arg_min(self) -> Self {
         let options = FunctionOptions {
             collect_groups: ApplyOptions::GroupWise,
-            returns_scalar: true,
+            flags: FunctionFlags::default() | FunctionFlags::RETURNS_SCALAR,
             fmt_str: "arg_min",
             ..Default::default()
         };
@@ -322,7 +333,7 @@ impl Expr {
     pub fn arg_max(self) -> Self {
         let options = FunctionOptions {
             collect_groups: ApplyOptions::GroupWise,
-            returns_scalar: true,
+            flags: FunctionFlags::default() | FunctionFlags::RETURNS_SCALAR,
             fmt_str: "arg_max",
             ..Default::default()
         };
@@ -331,7 +342,7 @@ impl Expr {
             move |s: Series| {
                 Ok(Some(Series::new(
                     s.name(),
-                    &[s.arg_max().map(|idx| idx as u32)],
+                    &[s.arg_max().map(|idx| idx as IdxSize)],
                 )))
             },
             GetOutput::from_type(IDX_DTYPE),
@@ -363,9 +374,9 @@ impl Expr {
             function: FunctionExpr::SearchSorted(side),
             options: FunctionOptions {
                 collect_groups: ApplyOptions::GroupWise,
-                returns_scalar: true,
+                flags: FunctionFlags::default() | FunctionFlags::RETURNS_SCALAR,
                 fmt_str: "search_sorted",
-                cast_to_supertypes: true,
+                cast_to_supertypes: Some(Default::default()),
                 ..Default::default()
             },
         }
@@ -377,7 +388,7 @@ impl Expr {
         Expr::Cast {
             expr: Arc::new(self),
             data_type,
-            strict: true,
+            options: CastOptions::Strict,
         }
     }
 
@@ -386,7 +397,16 @@ impl Expr {
         Expr::Cast {
             expr: Arc::new(self),
             data_type,
-            strict: false,
+            options: CastOptions::NonStrict,
+        }
+    }
+
+    /// Cast expression to another data type.
+    pub fn cast_with_options(self, data_type: DataType, cast_options: CastOptions) -> Self {
+        Expr::Cast {
+            expr: Arc::new(self),
+            data_type,
+            options: cast_options,
         }
     }
 
@@ -449,7 +469,22 @@ impl Expr {
     /// This has time complexity `O(n + k log(n))`.
     #[cfg(feature = "top_k")]
     pub fn top_k(self, k: Expr) -> Self {
-        self.apply_many_private(FunctionExpr::TopK(false), &[k], false, false)
+        self.apply_many_private(FunctionExpr::TopK { descending: false }, &[k], false, false)
+    }
+
+    /// Returns the `k` largest rows by given column.
+    ///
+    /// For single column, use [`Expr::top_k`].
+    #[cfg(feature = "top_k")]
+    pub fn top_k_by<K: Into<Expr>, E: AsRef<[IE]>, IE: Into<Expr> + Clone>(
+        self,
+        k: K,
+        by: E,
+        descending: Vec<bool>,
+    ) -> Self {
+        let mut args = vec![k.into()];
+        args.extend(by.as_ref().iter().map(|e| -> Expr { e.clone().into() }));
+        self.apply_many_private(FunctionExpr::TopKBy { descending }, &args, false, false)
     }
 
     /// Returns the `k` smallest elements.
@@ -457,7 +492,24 @@ impl Expr {
     /// This has time complexity `O(n + k log(n))`.
     #[cfg(feature = "top_k")]
     pub fn bottom_k(self, k: Expr) -> Self {
-        self.apply_many_private(FunctionExpr::TopK(true), &[k], false, false)
+        self.apply_many_private(FunctionExpr::TopK { descending: true }, &[k], false, false)
+    }
+
+    /// Returns the `k` smallest rows by given column.
+    ///
+    /// For single column, use [`Expr::bottom_k`].
+    // #[cfg(feature = "top_k")]
+    #[cfg(feature = "top_k")]
+    pub fn bottom_k_by<K: Into<Expr>, E: AsRef<[IE]>, IE: Into<Expr> + Clone>(
+        self,
+        k: K,
+        by: E,
+        descending: Vec<bool>,
+    ) -> Self {
+        let mut args = vec![k.into()];
+        args.extend(by.as_ref().iter().map(|e| -> Expr { e.clone().into() }));
+        let descending = descending.into_iter().map(|x| !x).collect();
+        self.apply_many_private(FunctionExpr::TopKBy { descending }, &args, false, false)
     }
 
     /// Reverse column
@@ -487,6 +539,7 @@ impl Expr {
             options: FunctionOptions {
                 collect_groups: ApplyOptions::ElementWise,
                 fmt_str: "map",
+                flags: FunctionFlags::default() | FunctionFlags::OPTIONAL_RE_ENTRANT,
                 ..Default::default()
             },
         }
@@ -641,12 +694,23 @@ impl Expr {
         input.push(self);
         input.extend_from_slice(arguments);
 
+        let cast_to_supertypes = if cast_to_supertypes {
+            Some(Default::default())
+        } else {
+            None
+        };
+
+        let mut flags = FunctionFlags::default();
+        if returns_scalar {
+            flags |= FunctionFlags::RETURNS_SCALAR;
+        }
+
         Expr::Function {
             input,
             function: function_expr,
             options: FunctionOptions {
                 collect_groups: ApplyOptions::GroupWise,
-                returns_scalar,
+                flags,
                 cast_to_supertypes,
                 ..Default::default()
             },
@@ -664,12 +728,22 @@ impl Expr {
         input.push(self);
         input.extend_from_slice(arguments);
 
+        let cast_to_supertypes = if cast_to_supertypes {
+            Some(Default::default())
+        } else {
+            None
+        };
+        let mut flags = FunctionFlags::default();
+        if returns_scalar {
+            flags |= FunctionFlags::RETURNS_SCALAR;
+        }
+
         Expr::Function {
             input,
             function: function_expr,
             options: FunctionOptions {
                 collect_groups: ApplyOptions::ElementWise,
-                returns_scalar,
+                flags,
                 cast_to_supertypes,
                 ..Default::default()
             },
@@ -747,21 +821,21 @@ impl Expr {
     pub fn product(self) -> Self {
         let options = FunctionOptions {
             collect_groups: ApplyOptions::GroupWise,
-            returns_scalar: true,
+            flags: FunctionFlags::default() | FunctionFlags::RETURNS_SCALAR,
             fmt_str: "product",
             ..Default::default()
         };
 
         self.function_with_options(
-            move |s: Series| Some(s.product()).transpose(),
+            move |s: Series| Some(s.product().map(|sc| sc.into_series(s.name()))).transpose(),
             GetOutput::map_dtype(|dt| {
-                use DataType::*;
-                match dt {
-                    Float32 => Float32,
-                    Float64 => Float64,
-                    UInt64 => UInt64,
-                    _ => Int64,
-                }
+                use DataType as T;
+                Ok(match dt {
+                    T::Float32 => T::Float32,
+                    T::Float64 => T::Float64,
+                    T::UInt64 => T::UInt64,
+                    _ => T::Int64,
+                })
             }),
             options,
         )
@@ -906,12 +980,13 @@ impl Expr {
     /// ╰────────┴────────╯
     /// ```
     pub fn over<E: AsRef<[IE]>, IE: Into<Expr> + Clone>(self, partition_by: E) -> Self {
-        self.over_with_options(partition_by, Default::default())
+        self.over_with_options(partition_by, None, Default::default())
     }
 
     pub fn over_with_options<E: AsRef<[IE]>, IE: Into<Expr> + Clone>(
         self,
         partition_by: E,
+        order_by: Option<(E, SortOptions)>,
         options: WindowMapping,
     ) -> Self {
         let partition_by = partition_by
@@ -919,9 +994,24 @@ impl Expr {
             .iter()
             .map(|e| e.clone().into())
             .collect();
+
+        let order_by = order_by.map(|(e, options)| {
+            let e = e.as_ref();
+            let e = if e.len() == 1 {
+                Arc::new(e[0].clone().into())
+            } else {
+                feature_gated!["dtype-struct", {
+                    let e = e.iter().map(|e| e.clone().into()).collect::<Vec<_>>();
+                    Arc::new(as_struct(e))
+                }]
+            };
+            (e, options)
+        });
+
         Expr::Window {
             function: Arc::new(self),
             partition_by,
+            order_by,
             options: options.into(),
         }
     }
@@ -934,6 +1024,7 @@ impl Expr {
         Expr::Window {
             function: Arc::new(self),
             partition_by: vec![index_col],
+            order_by: None,
             options: WindowType::Rolling(options),
         }
     }
@@ -943,14 +1034,10 @@ impl Expr {
 
         Expr::Function {
             input,
-            // super type will be replaced by type coercion
-            function: FunctionExpr::FillNull {
-                // will be set by `type_coercion`.
-                super_type: DataType::Unknown,
-            },
+            function: FunctionExpr::FillNull,
             options: FunctionOptions {
                 collect_groups: ApplyOptions::ElementWise,
-                cast_to_supertypes: true,
+                cast_to_supertypes: Some(Default::default()),
                 ..Default::default()
             },
         }
@@ -970,7 +1057,7 @@ impl Expr {
         // we take the not branch so that self is truthy value of `when -> then -> otherwise`
         // and that ensure we keep the name of `self`
 
-        when(self.clone().is_not_nan())
+        when(self.clone().is_not_nan().or(self.clone().is_null()))
             .then(self)
             .otherwise(fill_value.into())
     }
@@ -1015,7 +1102,7 @@ impl Expr {
     pub fn approx_n_unique(self) -> Self {
         self.apply_private(FunctionExpr::ApproxNUnique)
             .with_function_options(|mut options| {
-                options.returns_scalar = true;
+                options.flags |= FunctionFlags::RETURNS_SCALAR;
                 options
             })
     }
@@ -1195,67 +1282,134 @@ impl Expr {
         self.apply_private(FunctionExpr::Interpolate(method))
     }
 
+    #[cfg(feature = "rolling_window_by")]
+    #[allow(clippy::type_complexity)]
+    fn finish_rolling_by(
+        self,
+        by: Expr,
+        options: RollingOptionsDynamicWindow,
+        rolling_function_by: fn(RollingOptionsDynamicWindow) -> RollingFunctionBy,
+    ) -> Expr {
+        self.apply_many_private(
+            FunctionExpr::RollingExprBy(rolling_function_by(options)),
+            &[by],
+            false,
+            false,
+        )
+    }
+
+    #[cfg(feature = "interpolate_by")]
+    /// Fill null values using interpolation.
+    pub fn interpolate_by(self, by: Expr) -> Expr {
+        self.apply_many_private(FunctionExpr::InterpolateBy, &[by], false, false)
+    }
+
     #[cfg(feature = "rolling_window")]
     #[allow(clippy::type_complexity)]
     fn finish_rolling(
         self,
-        options: RollingOptions,
-        rolling_function: fn(RollingOptions) -> RollingFunction,
-        rolling_function_by: fn(RollingOptions) -> RollingFunction,
+        options: RollingOptionsFixedWindow,
+        rolling_function: fn(RollingOptionsFixedWindow) -> RollingFunction,
     ) -> Expr {
-        if let Some(ref by) = options.by {
-            let name = by.clone();
-            self.apply_many_private(
-                FunctionExpr::RollingExpr(rolling_function_by(options)),
-                &[col(&name)],
-                false,
-                false,
-            )
-        } else {
-            if !options.window_size.parsed_int {
-                panic!("if dynamic windows are used in a rolling aggregation, the 'by' argument must be set")
-            }
-            self.apply_private(FunctionExpr::RollingExpr(rolling_function(options)))
-        }
+        self.apply_private(FunctionExpr::RollingExpr(rolling_function(options)))
+    }
+
+    /// Apply a rolling minimum based on another column.
+    #[cfg(feature = "rolling_window_by")]
+    pub fn rolling_min_by(self, by: Expr, options: RollingOptionsDynamicWindow) -> Expr {
+        self.finish_rolling_by(by, options, RollingFunctionBy::MinBy)
+    }
+
+    /// Apply a rolling maximum based on another column.
+    #[cfg(feature = "rolling_window_by")]
+    pub fn rolling_max_by(self, by: Expr, options: RollingOptionsDynamicWindow) -> Expr {
+        self.finish_rolling_by(by, options, RollingFunctionBy::MaxBy)
+    }
+
+    /// Apply a rolling mean based on another column.
+    #[cfg(feature = "rolling_window_by")]
+    pub fn rolling_mean_by(self, by: Expr, options: RollingOptionsDynamicWindow) -> Expr {
+        self.finish_rolling_by(by, options, RollingFunctionBy::MeanBy)
+    }
+
+    /// Apply a rolling sum based on another column.
+    #[cfg(feature = "rolling_window_by")]
+    pub fn rolling_sum_by(self, by: Expr, options: RollingOptionsDynamicWindow) -> Expr {
+        self.finish_rolling_by(by, options, RollingFunctionBy::SumBy)
+    }
+
+    /// Apply a rolling quantile based on another column.
+    #[cfg(feature = "rolling_window_by")]
+    pub fn rolling_quantile_by(
+        self,
+        by: Expr,
+        interpol: QuantileInterpolOptions,
+        quantile: f64,
+        mut options: RollingOptionsDynamicWindow,
+    ) -> Expr {
+        options.fn_params = Some(Arc::new(RollingQuantileParams {
+            prob: quantile,
+            interpol,
+        }) as Arc<dyn Any + Send + Sync>);
+
+        self.finish_rolling_by(by, options, RollingFunctionBy::QuantileBy)
+    }
+
+    /// Apply a rolling variance based on another column.
+    #[cfg(feature = "rolling_window_by")]
+    pub fn rolling_var_by(self, by: Expr, options: RollingOptionsDynamicWindow) -> Expr {
+        self.finish_rolling_by(by, options, RollingFunctionBy::VarBy)
+    }
+
+    /// Apply a rolling std-dev based on another column.
+    #[cfg(feature = "rolling_window_by")]
+    pub fn rolling_std_by(self, by: Expr, options: RollingOptionsDynamicWindow) -> Expr {
+        self.finish_rolling_by(by, options, RollingFunctionBy::StdBy)
+    }
+
+    /// Apply a rolling median based on another column.
+    #[cfg(feature = "rolling_window_by")]
+    pub fn rolling_median_by(self, by: Expr, options: RollingOptionsDynamicWindow) -> Expr {
+        self.rolling_quantile_by(by, QuantileInterpolOptions::Linear, 0.5, options)
     }
 
     /// Apply a rolling minimum.
     ///
     /// See: [`RollingAgg::rolling_min`]
     #[cfg(feature = "rolling_window")]
-    pub fn rolling_min(self, options: RollingOptions) -> Expr {
-        self.finish_rolling(options, RollingFunction::Min, RollingFunction::MinBy)
+    pub fn rolling_min(self, options: RollingOptionsFixedWindow) -> Expr {
+        self.finish_rolling(options, RollingFunction::Min)
     }
 
     /// Apply a rolling maximum.
     ///
     /// See: [`RollingAgg::rolling_max`]
     #[cfg(feature = "rolling_window")]
-    pub fn rolling_max(self, options: RollingOptions) -> Expr {
-        self.finish_rolling(options, RollingFunction::Max, RollingFunction::MaxBy)
+    pub fn rolling_max(self, options: RollingOptionsFixedWindow) -> Expr {
+        self.finish_rolling(options, RollingFunction::Max)
     }
 
     /// Apply a rolling mean.
     ///
     /// See: [`RollingAgg::rolling_mean`]
     #[cfg(feature = "rolling_window")]
-    pub fn rolling_mean(self, options: RollingOptions) -> Expr {
-        self.finish_rolling(options, RollingFunction::Mean, RollingFunction::MeanBy)
+    pub fn rolling_mean(self, options: RollingOptionsFixedWindow) -> Expr {
+        self.finish_rolling(options, RollingFunction::Mean)
     }
 
     /// Apply a rolling sum.
     ///
     /// See: [`RollingAgg::rolling_sum`]
     #[cfg(feature = "rolling_window")]
-    pub fn rolling_sum(self, options: RollingOptions) -> Expr {
-        self.finish_rolling(options, RollingFunction::Sum, RollingFunction::SumBy)
+    pub fn rolling_sum(self, options: RollingOptionsFixedWindow) -> Expr {
+        self.finish_rolling(options, RollingFunction::Sum)
     }
 
     /// Apply a rolling median.
     ///
     /// See: [`RollingAgg::rolling_median`]
     #[cfg(feature = "rolling_window")]
-    pub fn rolling_median(self, options: RollingOptions) -> Expr {
+    pub fn rolling_median(self, options: RollingOptionsFixedWindow) -> Expr {
         self.rolling_quantile(QuantileInterpolOptions::Linear, 0.5, options)
     }
 
@@ -1267,30 +1421,26 @@ impl Expr {
         self,
         interpol: QuantileInterpolOptions,
         quantile: f64,
-        mut options: RollingOptions,
+        mut options: RollingOptionsFixedWindow,
     ) -> Expr {
         options.fn_params = Some(Arc::new(RollingQuantileParams {
             prob: quantile,
             interpol,
         }) as Arc<dyn Any + Send + Sync>);
 
-        self.finish_rolling(
-            options,
-            RollingFunction::Quantile,
-            RollingFunction::QuantileBy,
-        )
+        self.finish_rolling(options, RollingFunction::Quantile)
     }
 
     /// Apply a rolling variance.
     #[cfg(feature = "rolling_window")]
-    pub fn rolling_var(self, options: RollingOptions) -> Expr {
-        self.finish_rolling(options, RollingFunction::Var, RollingFunction::VarBy)
+    pub fn rolling_var(self, options: RollingOptionsFixedWindow) -> Expr {
+        self.finish_rolling(options, RollingFunction::Var)
     }
 
     /// Apply a rolling std-dev.
     #[cfg(feature = "rolling_window")]
-    pub fn rolling_std(self, options: RollingOptions) -> Expr {
-        self.finish_rolling(options, RollingFunction::Std, RollingFunction::StdBy)
+    pub fn rolling_std(self, options: RollingOptionsFixedWindow) -> Expr {
+        self.finish_rolling(options, RollingFunction::Std)
     }
 
     /// Apply a rolling skew.
@@ -1348,10 +1498,12 @@ impl Expr {
                     Ok(Some(out))
                 }
             },
-            GetOutput::map_field(|field| match field.data_type() {
-                DataType::Float64 => field.clone(),
-                DataType::Float32 => Field::new(field.name(), DataType::Float32),
-                _ => Field::new(field.name(), DataType::Float64),
+            GetOutput::map_field(|field| {
+                Ok(match field.data_type() {
+                    DataType::Float64 => field.clone(),
+                    DataType::Float32 => Field::new(field.name(), DataType::Float32),
+                    _ => Field::new(field.name(), DataType::Float64),
+                })
             }),
         )
         .with_fmt("rolling_map_float")
@@ -1375,7 +1527,25 @@ impl Expr {
 
     #[cfg(feature = "replace")]
     /// Replace the given values with other values.
-    pub fn replace<E: Into<Expr>>(
+    pub fn replace<E: Into<Expr>>(self, old: E, new: E) -> Expr {
+        let old = old.into();
+        let new = new.into();
+
+        // If we search and replace by literals, we can run on batches.
+        let literal_searchers = matches!(&old, Expr::Literal(_)) & matches!(&new, Expr::Literal(_));
+
+        let args = [old, new];
+
+        if literal_searchers {
+            self.map_many_private(FunctionExpr::Replace, &args, false, false)
+        } else {
+            self.apply_many_private(FunctionExpr::Replace, &args, false, false)
+        }
+    }
+
+    #[cfg(feature = "replace")]
+    /// Replace the given values with other values.
+    pub fn replace_strict<E: Into<Expr>>(
         self,
         old: E,
         new: E,
@@ -1384,7 +1554,8 @@ impl Expr {
     ) -> Expr {
         let old = old.into();
         let new = new.into();
-        // If we search and replace by literals, we can run on batches.
+
+        // If we replace by literals, we can run on batches.
         let literal_searchers = matches!(&old, Expr::Literal(_)) & matches!(&new, Expr::Literal(_));
 
         let mut args = vec![old, new];
@@ -1393,9 +1564,19 @@ impl Expr {
         }
 
         if literal_searchers {
-            self.map_many_private(FunctionExpr::Replace { return_dtype }, &args, false, false)
+            self.map_many_private(
+                FunctionExpr::ReplaceStrict { return_dtype },
+                &args,
+                false,
+                false,
+            )
         } else {
-            self.apply_many_private(FunctionExpr::Replace { return_dtype }, &args, false, false)
+            self.apply_many_private(
+                FunctionExpr::ReplaceStrict { return_dtype },
+                &args,
+                false,
+                false,
+            )
         }
     }
 
@@ -1415,7 +1596,7 @@ impl Expr {
             include_breaks,
         })
         .with_function_options(|mut opt| {
-            opt.pass_name_to_apply = true;
+            opt.flags |= FunctionFlags::PASS_NAME_TO_APPLY;
             opt
         })
     }
@@ -1438,7 +1619,7 @@ impl Expr {
             include_breaks,
         })
         .with_function_options(|mut opt| {
-            opt.pass_name_to_apply = true;
+            opt.flags |= FunctionFlags::PASS_NAME_TO_APPLY;
             opt
         })
     }
@@ -1462,7 +1643,7 @@ impl Expr {
             include_breaks,
         })
         .with_function_options(|mut opt| {
-            opt.pass_name_to_apply = true;
+            opt.flags |= FunctionFlags::PASS_NAME_TO_APPLY;
             opt
         })
     }
@@ -1504,7 +1685,7 @@ impl Expr {
     pub fn skew(self, bias: bool) -> Expr {
         self.apply_private(FunctionExpr::Skew(bias))
             .with_function_options(|mut options| {
-                options.returns_scalar = true;
+                options.flags |= FunctionFlags::RETURNS_SCALAR;
                 options
             })
     }
@@ -1520,7 +1701,7 @@ impl Expr {
     pub fn kurtosis(self, fisher: bool, bias: bool) -> Expr {
         self.apply_private(FunctionExpr::Kurtosis(fisher, bias))
             .with_function_options(|mut options| {
-                options.returns_scalar = true;
+                options.flags |= FunctionFlags::RETURNS_SCALAR;
                 options
             })
     }
@@ -1535,9 +1716,9 @@ impl Expr {
         self.map_private(FunctionExpr::LowerBound)
     }
 
-    pub fn reshape(self, dimensions: &[i64]) -> Self {
+    pub fn reshape(self, dimensions: &[i64], nested_type: NestedType) -> Self {
         let dimensions = dimensions.to_vec();
-        self.apply_private(FunctionExpr::Reshape(dimensions))
+        self.apply_private(FunctionExpr::Reshape(dimensions, nested_type))
     }
 
     #[cfg(feature = "ewma")]
@@ -1548,12 +1729,9 @@ impl Expr {
 
     #[cfg(feature = "ewma_by")]
     /// Calculate the exponentially-weighted moving average by a time column.
-    pub fn ewm_mean_by(self, times: Expr, half_life: Duration, check_sorted: bool) -> Self {
+    pub fn ewm_mean_by(self, times: Expr, half_life: Duration) -> Self {
         self.apply_many_private(
-            FunctionExpr::EwmMeanBy {
-                half_life,
-                check_sorted,
-            },
+            FunctionExpr::EwmMeanBy { half_life },
             &[times],
             false,
             false,
@@ -1582,7 +1760,7 @@ impl Expr {
     pub fn any(self, ignore_nulls: bool) -> Self {
         self.apply_private(BooleanFunction::Any { ignore_nulls }.into())
             .with_function_options(|mut opt| {
-                opt.returns_scalar = true;
+                opt.flags |= FunctionFlags::RETURNS_SCALAR;
                 opt
             })
     }
@@ -1597,7 +1775,7 @@ impl Expr {
     pub fn all(self, ignore_nulls: bool) -> Self {
         self.apply_private(BooleanFunction::All { ignore_nulls }.into())
             .with_function_options(|mut opt| {
-                opt.returns_scalar = true;
+                opt.flags |= FunctionFlags::RETURNS_SCALAR;
                 opt
             })
     }
@@ -1606,18 +1784,23 @@ impl Expr {
     /// needed to fit the extrema of this [`Series`].
     /// This can be used to reduce memory pressure.
     pub fn shrink_dtype(self) -> Self {
-        self.map_private(FunctionExpr::ShrinkType)
+        self.apply_private(FunctionExpr::ShrinkType)
     }
 
     #[cfg(feature = "dtype-struct")]
     /// Count all unique values and create a struct mapping value to count.
     /// (Note that it is better to turn parallel off in the aggregation context).
-    pub fn value_counts(self, sort: bool, parallel: bool) -> Self {
-        self.apply_private(FunctionExpr::ValueCounts { sort, parallel })
-            .with_function_options(|mut opts| {
-                opts.pass_name_to_apply = true;
-                opts
-            })
+    pub fn value_counts(self, sort: bool, parallel: bool, name: String, normalize: bool) -> Self {
+        self.apply_private(FunctionExpr::ValueCounts {
+            sort,
+            parallel,
+            name,
+            normalize,
+        })
+        .with_function_options(|mut opts| {
+            opts.flags |= FunctionFlags::PASS_NAME_TO_APPLY;
+            opts
+        })
     }
 
     #[cfg(feature = "unique_counts")]
@@ -1652,7 +1835,7 @@ impl Expr {
     pub fn entropy(self, base: f64, normalize: bool) -> Self {
         self.apply_private(FunctionExpr::Entropy { base, normalize })
             .with_function_options(|mut options| {
-                options.returns_scalar = true;
+                options.flags |= FunctionFlags::RETURNS_SCALAR;
                 options
             })
     }
@@ -1660,7 +1843,7 @@ impl Expr {
     pub fn null_count(self) -> Expr {
         self.apply_private(FunctionExpr::NullCount)
             .with_function_options(|mut options| {
-                options.returns_scalar = true;
+                options.flags |= FunctionFlags::RETURNS_SCALAR;
                 options
             })
     }
@@ -1798,8 +1981,8 @@ where
         output_type,
         options: FunctionOptions {
             collect_groups: ApplyOptions::ApplyList,
-            returns_scalar: true,
             fmt_str: "",
+            flags: FunctionFlags::default() | FunctionFlags::RETURNS_SCALAR,
             ..Default::default()
         },
     }
@@ -1825,6 +2008,10 @@ where
     E: AsRef<[Expr]>,
 {
     let input = expr.as_ref().to_vec();
+    let mut flags = FunctionFlags::default();
+    if returns_scalar {
+        flags |= FunctionFlags::RETURNS_SCALAR;
+    }
 
     Expr::AnonymousFunction {
         input,
@@ -1834,8 +2021,8 @@ where
             collect_groups: ApplyOptions::GroupWise,
             // don't set this to true
             // this is for the caller to decide
-            returns_scalar,
             fmt_str: "",
+            flags,
             ..Default::default()
         },
     }
@@ -1846,12 +2033,17 @@ pub fn len() -> Expr {
     Expr::Len
 }
 
-/// First column in DataFrame.
+/// First column in a DataFrame.
 pub fn first() -> Expr {
     Expr::Nth(0)
 }
 
-/// Last column in DataFrame.
+/// Last column in a DataFrame.
 pub fn last() -> Expr {
     Expr::Nth(-1)
 }
+
+/// Nth column in a DataFrame.
+pub fn nth(n: i64) -> Expr {
+    Expr::Nth(n)
+}
