@@ -33,10 +33,12 @@ impl StringNameSpace {
     }
 
     /// Uses aho-corasick to find many patterns.
+    ///
     /// # Arguments
     /// - `patterns`: an expression that evaluates to an String column
     /// - `ascii_case_insensitive`: Enable ASCII-aware case insensitive matching.
-    ///  When this option is enabled, searching will be performed without respect to case for ASCII letters (a-z and A-Z) only.
+    ///   When this option is enabled, searching will be performed without respect to case for
+    ///   ASCII letters (a-z and A-Z) only.
     #[cfg(feature = "find_many")]
     pub fn contains_any(self, patterns: Expr, ascii_case_insensitive: bool) -> Expr {
         self.0.map_many_private(
@@ -51,10 +53,11 @@ impl StringNameSpace {
 
     /// Uses aho-corasick to replace many patterns.
     /// # Arguments
-    /// - `patterns`: an expression that evaluates to an String column
-    /// - `replace_with`: an expression that evaluates to an String column
-    /// - `ascii_case_insensitive`: Enable ASCII-aware case insensitive matching.
-    ///  When this option is enabled, searching will be performed without respect to case for ASCII letters (a-z and A-Z) only.
+    /// - `patterns`: an expression that evaluates to a String column
+    /// - `replace_with`: an expression that evaluates to a String column
+    /// - `ascii_case_insensitive`: Enable ASCII-aware case-insensitive matching.
+    ///   When this option is enabled, searching will be performed without respect to case for
+    ///   ASCII letters (a-z and A-Z) only.
     #[cfg(feature = "find_many")]
     pub fn replace_many(
         self,
@@ -72,6 +75,31 @@ impl StringNameSpace {
         )
     }
 
+    /// Uses aho-corasick to replace many patterns.
+    /// # Arguments
+    /// - `patterns`: an expression that evaluates to a String column
+    /// - `ascii_case_insensitive`: Enable ASCII-aware case-insensitive matching.
+    ///   When this option is enabled, searching will be performed without respect to case for
+    ///   ASCII letters (a-z and A-Z) only.
+    /// - `overlapping`: Whether matches may overlap.
+    #[cfg(feature = "find_many")]
+    pub fn extract_many(
+        self,
+        patterns: Expr,
+        ascii_case_insensitive: bool,
+        overlapping: bool,
+    ) -> Expr {
+        self.0.map_many_private(
+            FunctionExpr::StringExpr(StringFunction::ExtractMany {
+                ascii_case_insensitive,
+                overlapping,
+            }),
+            &[patterns],
+            false,
+            false,
+        )
+    }
+
     /// Check if a string value ends with the `sub` string.
     pub fn ends_with(self, sub: Expr) -> Expr {
         self.0.map_many_private(
@@ -209,7 +237,7 @@ impl StringNameSpace {
         )
     }
 
-    /// Find the index of a substring defined by a regular expressons within another string value.
+    /// Find the index of a substring defined by a regular expressions within another string value.
     #[cfg(feature = "regex")]
     pub fn find(self, pat: Expr, strict: bool) -> Expr {
         self.0.map_many_private(
@@ -269,11 +297,7 @@ impl StringNameSpace {
         let time_unit = match (&options.format, time_unit) {
             (_, Some(time_unit)) => time_unit,
             (Some(format), None) => {
-                if format.contains("%.9f")
-                    || format.contains("%9f")
-                    || format.contains("%f")
-                    || format.contains("%.f")
-                {
+                if format.contains("%.9f") || format.contains("%9f") {
                     TimeUnit::Nanoseconds
                 } else if format.contains("%.3f") || format.contains("%3f") {
                     TimeUnit::Milliseconds
@@ -305,7 +329,7 @@ impl StringNameSpace {
     ///
     /// * `delimiter` - A string that will act as delimiter between values.
     #[cfg(feature = "concat_str")]
-    pub fn concat(self, delimiter: &str, ignore_nulls: bool) -> Expr {
+    pub fn join(self, delimiter: &str, ignore_nulls: bool) -> Expr {
         self.0
             .apply_private(
                 StringFunction::ConcatVertical {
@@ -315,7 +339,7 @@ impl StringNameSpace {
                 .into(),
             )
             .with_function_options(|mut options| {
-                options.returns_scalar = true;
+                options.flags |= FunctionFlags::RETURNS_SCALAR;
                 options.collect_groups = ApplyOptions::GroupWise;
                 options
             })
@@ -547,15 +571,11 @@ impl StringNameSpace {
         )
     }
 
-    pub fn explode(self) -> Expr {
-        self.0
-            .apply_private(FunctionExpr::StringExpr(StringFunction::Explode))
-    }
-
     #[cfg(feature = "extract_jsonpath")]
     pub fn json_decode(self, dtype: Option<DataType>, infer_schema_len: Option<usize>) -> Expr {
+        // Apply, because dtype should be inferred only once and be consistent over chunks/morsels.
         self.0
-            .map_private(FunctionExpr::StringExpr(StringFunction::JsonDecode {
+            .apply_private(FunctionExpr::StringExpr(StringFunction::JsonDecode {
                 dtype,
                 infer_schema_len,
             }))
