@@ -1,10 +1,14 @@
+mod read;
+mod read_indexes;
+mod write;
+
 use std::io::{Cursor, Read, Seek};
 
 use arrow::array::*;
 use arrow::bitmap::Bitmap;
 use arrow::datatypes::*;
 use arrow::legacy::prelude::LargeListArray;
-use arrow::record_batch::RecordBatch;
+use arrow::record_batch::RecordBatchT;
 use arrow::types::{i256, NativeType};
 use ethnum::AsI256;
 use polars_error::PolarsResult;
@@ -12,15 +16,6 @@ use polars_parquet::read as p_read;
 use polars_parquet::read::statistics::*;
 use polars_parquet::write::*;
 
-#[cfg(feature = "io_json_integration")]
-mod integration;
-mod read;
-mod read_indexes;
-mod write;
-
-#[cfg(feature = "io_parquet_sample_test")]
-mod sample_tests;
-
 type ArrayStats = (Box<dyn Array>, Statistics);
 
 fn new_struct(
@@ -58,7 +53,7 @@ pub fn read_column<R: Read + Seek>(mut reader: R, column: &str) -> PolarsResult<
 
     let statistics = deserialize(field, row_group)?;
 
-    let mut reader = p_read::FileReader::new(reader, metadata.row_groups, schema, None, None, None);
+    let mut reader = p_read::FileReader::new(reader, metadata.row_groups, schema, None, None);
 
     let array = reader.next().unwrap()?.into_arrays().pop().unwrap();
 
@@ -569,7 +564,7 @@ pub fn pyarrow_nullable_statistics(column: &str) -> Statistics {
             max_value: Box::new(Utf8ViewArray::from_slice([Some("def")])),
         },
         "bool" => Statistics {
-            distinct_count: UInt64Array::from([None]).boxed(),
+            distinct_count: UInt64Array::from([Some(2)]).boxed(),
             null_count: UInt64Array::from([Some(4)]).boxed(),
             min_value: Box::new(BooleanArray::from_slice([false])),
             max_value: Box::new(BooleanArray::from_slice([true])),
@@ -706,7 +701,7 @@ pub fn pyarrow_nested_nullable_statistics(column: &str) -> Statistics {
             max_value: new_list(Box::new(Int16Array::from_slice([10])), true).boxed(),
         },
         "list_bool" => Statistics {
-            distinct_count: new_list(UInt64Array::from([None]).boxed(), true).boxed(),
+            distinct_count: new_list(UInt64Array::from([Some(2)]).boxed(), true).boxed(),
             null_count: new_list(UInt64Array::from([Some(1)]).boxed(), true).boxed(),
             min_value: new_list(Box::new(BooleanArray::from_slice([false])), true).boxed(),
             max_value: new_list(Box::new(BooleanArray::from_slice([true])), true).boxed(),
@@ -1101,7 +1096,7 @@ pub fn pyarrow_struct_statistics(column: &str) -> Statistics {
             distinct_count: new_struct(
                 vec![
                     Box::new(UInt64Array::from([None])),
-                    Box::new(UInt64Array::from([None])),
+                    Box::new(UInt64Array::from([Some(2)])),
                 ],
                 names.clone(),
             )
@@ -1135,7 +1130,7 @@ pub fn pyarrow_struct_statistics(column: &str) -> Statistics {
                     new_struct(
                         vec![
                             Box::new(UInt64Array::from([None])),
-                            Box::new(UInt64Array::from([None])),
+                            Box::new(UInt64Array::from([Some(2)])),
                         ],
                         names.clone(),
                     )
@@ -1259,13 +1254,13 @@ pub fn pyarrow_struct_statistics(column: &str) -> Statistics {
 
 fn integration_write(
     schema: &ArrowSchema,
-    chunks: &[RecordBatch<Box<dyn Array>>],
+    chunks: &[RecordBatchT<Box<dyn Array>>],
 ) -> PolarsResult<Vec<u8>> {
     let options = WriteOptions {
-        write_statistics: true,
+        statistics: StatisticsOptions::full(),
         compression: CompressionOptions::Uncompressed,
         version: Version::V1,
-        data_pagesize_limit: None,
+        data_page_size: None,
     };
 
     let encodings = schema
@@ -1297,7 +1292,7 @@ fn integration_write(
     Ok(writer.into_inner().into_inner())
 }
 
-type IntegrationRead = (ArrowSchema, Vec<RecordBatch<Box<dyn Array>>>);
+type IntegrationRead = (ArrowSchema, Vec<RecordBatchT<Box<dyn Array>>>);
 
 fn integration_read(data: &[u8], limit: Option<usize>) -> PolarsResult<IntegrationRead> {
     let mut reader = Cursor::new(data);
@@ -1312,7 +1307,6 @@ fn integration_read(data: &[u8], limit: Option<usize>) -> PolarsResult<Integrati
         Cursor::new(data),
         metadata.row_groups,
         schema.clone(),
-        None,
         limit,
         None,
     );
@@ -1322,7 +1316,7 @@ fn integration_read(data: &[u8], limit: Option<usize>) -> PolarsResult<Integrati
     Ok((schema, batches))
 }
 
-fn generic_data() -> PolarsResult<(ArrowSchema, RecordBatch<Box<dyn Array>>)> {
+fn generic_data() -> PolarsResult<(ArrowSchema, RecordBatchT<Box<dyn Array>>)> {
     let array1 = PrimitiveArray::<i64>::from([Some(1), None, Some(2)])
         .to(ArrowDataType::Duration(TimeUnit::Second));
     let array2 = Utf8ViewArray::from_slice([Some("a"), None, Some("bb")]);
@@ -1376,7 +1370,7 @@ fn generic_data() -> PolarsResult<(ArrowSchema, RecordBatch<Box<dyn Array>>)> {
         Field::new("a12", array12.data_type().clone(), true),
         Field::new("a13", array13.data_type().clone(), true),
     ]);
-    let chunk = RecordBatch::try_new(vec![
+    let chunk = RecordBatchT::try_new(vec![
         array1.boxed(),
         array2.boxed(),
         array3.boxed(),
@@ -1396,7 +1390,7 @@ fn generic_data() -> PolarsResult<(ArrowSchema, RecordBatch<Box<dyn Array>>)> {
 
 fn assert_roundtrip(
     schema: ArrowSchema,
-    chunk: RecordBatch<Box<dyn Array>>,
+    chunk: RecordBatchT<Box<dyn Array>>,
     limit: Option<usize>,
 ) -> PolarsResult<()> {
     let r = integration_write(&schema, &[chunk.clone()])?;
@@ -1409,7 +1403,7 @@ fn assert_roundtrip(
             .into_iter()
             .map(|x| x.sliced(0, limit))
             .collect::<Vec<_>>();
-        RecordBatch::new(expected)
+        RecordBatchT::new(expected)
     } else {
         chunk
     };
@@ -1473,7 +1467,7 @@ fn assert_array_roundtrip(
         array.data_type().clone(),
         is_nullable,
     )]);
-    let chunk = RecordBatch::try_new(vec![array])?;
+    let chunk = RecordBatchT::try_new(vec![array])?;
 
     assert_roundtrip(schema, chunk, limit)
 }
@@ -1581,7 +1575,7 @@ fn limit_list() -> PolarsResult<()> {
 
 fn nested_dict_data(
     data_type: ArrowDataType,
-) -> PolarsResult<(ArrowSchema, RecordBatch<Box<dyn Array>>)> {
+) -> PolarsResult<(ArrowSchema, RecordBatchT<Box<dyn Array>>)> {
     let values = match data_type {
         ArrowDataType::Float32 => PrimitiveArray::from_slice([1.0f32, 3.0]).boxed(),
         ArrowDataType::Utf8View => Utf8ViewArray::from_slice([Some("a"), Some("b")]).boxed(),
@@ -1602,7 +1596,7 @@ fn nested_dict_data(
     )?;
 
     let schema = ArrowSchema::from(vec![Field::new("c1", values.data_type().clone(), true)]);
-    let chunk = RecordBatch::try_new(vec![values.boxed()])?;
+    let chunk = RecordBatchT::try_new(vec![values.boxed()])?;
 
     Ok((schema, chunk))
 }
@@ -1630,8 +1624,8 @@ fn nested_dict_limit() -> PolarsResult<()> {
 
 #[test]
 fn filter_chunk() -> PolarsResult<()> {
-    let chunk1 = RecordBatch::new(vec![PrimitiveArray::from_slice([1i16, 3]).boxed()]);
-    let chunk2 = RecordBatch::new(vec![PrimitiveArray::from_slice([2i16, 4]).boxed()]);
+    let chunk1 = RecordBatchT::new(vec![PrimitiveArray::from_slice([1i16, 3]).boxed()]);
+    let chunk2 = RecordBatchT::new(vec![PrimitiveArray::from_slice([2i16, 4]).boxed()]);
     let schema = ArrowSchema::from(vec![Field::new("c1", ArrowDataType::Int16, true)]);
 
     let r = integration_write(&schema, &[chunk1.clone(), chunk2.clone()])?;
@@ -1652,7 +1646,7 @@ fn filter_chunk() -> PolarsResult<()> {
         .map(|(_, row_group)| row_group)
         .collect();
 
-    let reader = p_read::FileReader::new(reader, row_groups, schema, None, None, None);
+    let reader = p_read::FileReader::new(reader, row_groups, schema, None, None);
 
     let new_chunks = reader.collect::<PolarsResult<Vec<_>>>()?;
 
