@@ -1,4 +1,4 @@
-use polars_parquet::parquet::error::{Error, Result};
+use polars_parquet::parquet::error::{ParquetError, ParquetResult};
 use polars_parquet::parquet::types::{decode, NativeType};
 
 #[derive(Debug)]
@@ -16,9 +16,9 @@ impl<T: NativeType> PrimitivePageDict<T> {
     }
 
     #[inline]
-    pub fn value(&self, index: usize) -> Result<&T> {
+    pub fn value(&self, index: usize) -> ParquetResult<&T> {
         self.values.get(index).ok_or_else(|| {
-            Error::OutOfSpec(
+            ParquetError::OutOfSpec(
                 "The data page has an index larger than the dictionary page values".to_string(),
             )
         })
@@ -29,13 +29,13 @@ pub fn read<T: NativeType>(
     buf: &[u8],
     num_values: usize,
     _is_sorted: bool,
-) -> Result<PrimitivePageDict<T>> {
+) -> ParquetResult<PrimitivePageDict<T>> {
     let size_of = std::mem::size_of::<T>();
 
     let typed_size = num_values.wrapping_mul(size_of);
 
     let values = buf.get(..typed_size).ok_or_else(|| {
-        Error::OutOfSpec(
+        ParquetError::OutOfSpec(
             "The number of values declared in the dict page does not match the length of the page"
                 .to_string(),
         )
