@@ -2,7 +2,6 @@ from polars.functions.aggregation.horizontal import (
     all_horizontal,
     any_horizontal,
     cum_sum_horizontal,
-    cumsum_horizontal,
     max_horizontal,
     mean_horizontal,
     min_horizontal,
@@ -12,7 +11,6 @@ from polars.functions.aggregation.vertical import (
     all,
     any,
     cum_sum,
-    cumsum,
     max,
     min,
     sum,
@@ -22,14 +20,12 @@ __all__ = [
     "all",
     "any",
     "cum_sum",
-    "cumsum",
     "max",
     "min",
     "sum",
     "all_horizontal",
     "any_horizontal",
     "cum_sum_horizontal",
-    "cumsum_horizontal",
     "max_horizontal",
     "mean_horizontal",
     "min_horizontal",
