@@ -10,13 +10,14 @@ import numpy as np
 import pandas as pd
 import pyarrow as pa
 import pytest
+from packaging.version import parse as parse_version
 from pydantic import BaseModel, Field, TypeAdapter
 
 import polars as pl
 from polars._utils.construction.utils import try_get_type_hints
-from polars.datatypes import PolarsDataType, numpy_char_code_to_dtype
+from polars.datatypes import numpy_char_code_to_dtype
 from polars.dependencies import dataclasses, pydantic
-from polars.exceptions import TimeZoneAwareConstructorWarning
+from polars.exceptions import ShapeError
 from polars.testing import assert_frame_equal, assert_series_equal
 
 if TYPE_CHECKING:
@@ -24,7 +25,8 @@ if TYPE_CHECKING:
 
     from zoneinfo import ZoneInfo
 
-    from polars.datatypes import PolarsDataType
+    from polars._typing import PolarsDataType
+
 else:
     from polars._utils.convert import string_to_zoneinfo as ZoneInfo
 
@@ -175,20 +177,18 @@ def test_init_dict() -> None:
 
 
 def test_error_string_dtypes() -> None:
-    with pytest.raises(ValueError, match="cannot infer dtype"):
+    with pytest.raises(TypeError, match="cannot parse input"):
         pl.DataFrame(
             data={"x": [1, 2], "y": [3, 4], "z": [5, 6]},
             schema={"x": "i16", "y": "i32", "z": "f32"},  # type: ignore[dict-item]
         )
 
-    with pytest.raises(ValueError, match="not a valid Polars data type"):
+    with pytest.raises(TypeError, match="cannot parse input"):
         pl.Series("n", [1, 2, 3], dtype="f32")  # type: ignore[arg-type]
 
 
-def test_init_structured_objects(monkeypatch: Any) -> None:
+def test_init_structured_objects() -> None:
     # validate init from dataclass, namedtuple, and pydantic model objects
-    monkeypatch.setenv("POLARS_ACTIVATE_DECIMAL", "1")
-
     @dataclasses.dataclass
     class TradeDC:
         timestamp: datetime
@@ -327,118 +327,121 @@ def test_init_structured_objects_unhashable() -> None:
     assert df.rows() == test_data
 
 
-def test_init_structured_objects_nested() -> None:
-    for Foo, Bar, Baz in (
+@pytest.mark.parametrize(
+    ("foo", "bar", "baz"),
+    [
         (_TestFooDC, _TestBarDC, _TestBazDC),
         (_TestFooPD, _TestBarPD, _TestBazPD),
         (_TestFooNT, _TestBarNT, _TestBazNT),
-    ):
-        data = [
-            Foo(
-                x=100,
-                y=Bar(
-                    a="hello",
-                    b=800,
-                    c=Baz(d=datetime(2023, 4, 12, 10, 30), e=-10.5, f="world"),
-                ),
-            )
-        ]
-        df = pl.DataFrame(data)
-        # shape: (1, 2)
-        # ┌─────┬───────────────────────────────────┐
-        # │ x   ┆ y                                 │
-        # │ --- ┆ ---                               │
-        # │ i64 ┆ struct[3]                         │
-        # ╞═════╪═══════════════════════════════════╡
-        # │ 100 ┆ {"hello",800,{2023-04-12 10:30:0… │
-        # └─────┴───────────────────────────────────┘
-
-        assert df.schema == {
-            "x": pl.Int64,
-            "y": pl.Struct(
-                [
-                    pl.Field("a", pl.String),
-                    pl.Field("b", pl.Int64),
-                    pl.Field(
-                        "c",
-                        pl.Struct(
-                            [
-                                pl.Field("d", pl.Datetime("us")),
-                                pl.Field("e", pl.Float64),
-                                pl.Field("f", pl.String),
-                            ]
-                        ),
-                    ),
-                ]
+    ],
+)
+def test_init_structured_objects_nested(foo: Any, bar: Any, baz: Any) -> None:
+    data = [
+        foo(
+            x=100,
+            y=bar(
+                a="hello",
+                b=800,
+                c=baz(d=datetime(2023, 4, 12, 10, 30), e=-10.5, f="world"),
             ),
+        )
+    ]
+    df = pl.DataFrame(data)
+    # shape: (1, 2)
+    # ┌─────┬───────────────────────────────────┐
+    # │ x   ┆ y                                 │
+    # │ --- ┆ ---                               │
+    # │ i64 ┆ struct[3]                         │
+    # ╞═════╪═══════════════════════════════════╡
+    # │ 100 ┆ {"hello",800,{2023-04-12 10:30:0… │
+    # └─────┴───────────────────────────────────┘
+
+    assert df.schema == {
+        "x": pl.Int64,
+        "y": pl.Struct(
+            [
+                pl.Field("a", pl.String),
+                pl.Field("b", pl.Int64),
+                pl.Field(
+                    "c",
+                    pl.Struct(
+                        [
+                            pl.Field("d", pl.Datetime("us")),
+                            pl.Field("e", pl.Float64),
+                            pl.Field("f", pl.String),
+                        ]
+                    ),
+                ),
+            ]
+        ),
+    }
+    assert df.row(0) == (
+        100,
+        {
+            "a": "hello",
+            "b": 800,
+            "c": {
+                "d": datetime(2023, 4, 12, 10, 30),
+                "e": -10.5,
+                "f": "world",
+            },
+        },
+    )
+
+    # validate nested schema override
+    override_struct_schema: dict[str, PolarsDataType] = {
+        "x": pl.Int16,
+        "y": pl.Struct(
+            [
+                pl.Field("a", pl.String),
+                pl.Field("b", pl.Int32),
+                pl.Field(
+                    name="c",
+                    dtype=pl.Struct(
+                        [
+                            pl.Field("d", pl.Datetime("ms")),
+                            pl.Field("e", pl.Float32),
+                            pl.Field("f", pl.String),
+                        ]
+                    ),
+                ),
+            ]
+        ),
+    }
+    for schema, schema_overrides in (
+        (None, override_struct_schema),
+        (override_struct_schema, None),
+    ):
+        df = (
+            pl.DataFrame(data, schema=schema, schema_overrides=schema_overrides)
+            .unnest("y")
+            .unnest("c")
+        )
+        # shape: (1, 6)
+        # ┌─────┬───────┬─────┬─────────────────────┬───────┬───────┐
+        # │ x   ┆ a     ┆ b   ┆ d                   ┆ e     ┆ f     │
+        # │ --- ┆ ---   ┆ --- ┆ ---                 ┆ ---   ┆ ---   │
+        # │ i16 ┆ str   ┆ i32 ┆ datetime[ms]        ┆ f32   ┆ str   │
+        # ╞═════╪═══════╪═════╪═════════════════════╪═══════╪═══════╡
+        # │ 100 ┆ hello ┆ 800 ┆ 2023-04-12 10:30:00 ┆ -10.5 ┆ world │
+        # └─────┴───────┴─────┴─────────────────────┴───────┴───────┘
+        assert df.schema == {
+            "x": pl.Int16,
+            "a": pl.String,
+            "b": pl.Int32,
+            "d": pl.Datetime("ms"),
+            "e": pl.Float32,
+            "f": pl.String,
         }
         assert df.row(0) == (
             100,
-            {
-                "a": "hello",
-                "b": 800,
-                "c": {
-                    "d": datetime(2023, 4, 12, 10, 30),
-                    "e": -10.5,
-                    "f": "world",
-                },
-            },
+            "hello",
+            800,
+            datetime(2023, 4, 12, 10, 30),
+            -10.5,
+            "world",
         )
 
-        # validate nested schema override
-        override_struct_schema: dict[str, PolarsDataType] = {
-            "x": pl.Int16,
-            "y": pl.Struct(
-                [
-                    pl.Field("a", pl.String),
-                    pl.Field("b", pl.Int32),
-                    pl.Field(
-                        name="c",
-                        dtype=pl.Struct(
-                            [
-                                pl.Field("d", pl.Datetime("ms")),
-                                pl.Field("e", pl.Float32),
-                                pl.Field("f", pl.String),
-                            ]
-                        ),
-                    ),
-                ]
-            ),
-        }
-        for schema, schema_overrides in (
-            (None, override_struct_schema),
-            (override_struct_schema, None),
-        ):
-            df = (
-                pl.DataFrame(data, schema=schema, schema_overrides=schema_overrides)
-                .unnest("y")
-                .unnest("c")
-            )
-            # shape: (1, 6)
-            # ┌─────┬───────┬─────┬─────────────────────┬───────┬───────┐
-            # │ x   ┆ a     ┆ b   ┆ d                   ┆ e     ┆ f     │
-            # │ --- ┆ ---   ┆ --- ┆ ---                 ┆ ---   ┆ ---   │
-            # │ i16 ┆ str   ┆ i32 ┆ datetime[ms]        ┆ f32   ┆ str   │
-            # ╞═════╪═══════╪═════╪═════════════════════╪═══════╪═══════╡
-            # │ 100 ┆ hello ┆ 800 ┆ 2023-04-12 10:30:00 ┆ -10.5 ┆ world │
-            # └─────┴───────┴─────┴─────────────────────┴───────┴───────┘
-            assert df.schema == {
-                "x": pl.Int16,
-                "a": pl.String,
-                "b": pl.Int32,
-                "d": pl.Datetime("ms"),
-                "e": pl.Float32,
-                "f": pl.String,
-            }
-            assert df.row(0) == (
-                100,
-                "hello",
-                800,
-                datetime(2023, 4, 12, 10, 30),
-                -10.5,
-                "world",
-            )
-
 
 def test_dataclasses_initvar_typing() -> None:
     @dataclasses.dataclass
@@ -543,15 +546,20 @@ def test_init_ndarray() -> None:
     assert np.array_equal(df.to_numpy(), np.arange(4).reshape(-1, 1).astype(np.int64))
 
     df = pl.DataFrame(np.arange(4).reshape(-1, 2).astype(np.int64), schema=["a"])
-    assert_frame_equal(df, pl.DataFrame({"a": [[0, 1], [2, 3]]}))
+    assert_frame_equal(
+        df,
+        pl.DataFrame(
+            {"a": [[0, 1], [2, 3]]}, schema={"a": pl.Array(pl.Int64, shape=2)}
+        ),
+    )
 
     # 2D numpy arrays
     df = pl.DataFrame({"a": np.arange(5, dtype=np.int64).reshape(1, -1)})
-    assert df.dtypes == [pl.List(pl.Int64)]
+    assert df.dtypes == [pl.Array(pl.Int64, shape=5)]
     assert df.shape == (1, 1)
 
     df = pl.DataFrame({"a": np.arange(10, dtype=np.int64).reshape(2, -1)})
-    assert df.dtypes == [pl.List(pl.Int64)]
+    assert df.dtypes == [pl.Array(pl.Int64, shape=5)]
     assert df.shape == (2, 1)
     assert df.rows() == [([0, 1, 2, 3, 4],), ([5, 6, 7, 8, 9],)]
 
@@ -660,6 +668,7 @@ def test_init_numpy_scalars() -> None:
     df_expected = pl.from_records(
         data=[(True, 16, 1234), (False, 64, 9876)],
         schema=OrderedDict([("bool", pl.Boolean), ("i8", pl.Int8), ("u32", pl.UInt32)]),
+        orient="row",
     )
     assert_frame_equal(df, df_expected)
 
@@ -834,13 +843,14 @@ def test_init_py_dtype_misc_float() -> None:
 
 def test_init_seq_of_seq() -> None:
     # List of lists
-    df = pl.DataFrame([[1, 2, 3], [4, 5, 6]], schema=["a", "b", "c"])
+    df = pl.DataFrame([[1, 2, 3], [4, 5, 6]], schema=["a", "b", "c"], orient="row")
     expected = pl.DataFrame({"a": [1, 4], "b": [2, 5], "c": [3, 6]})
     assert_frame_equal(df, expected)
 
     df = pl.DataFrame(
         [[1, 2, 3], [4, 5, 6]],
         schema=[("a", pl.Int8), ("b", pl.Int16), ("c", pl.Int32)],
+        orient="row",
     )
     assert df.schema == {"a": pl.Int8, "b": pl.Int16, "c": pl.Int32}
     assert df.rows() == [(1, 2, 3), (4, 5, 6)]
@@ -894,21 +904,15 @@ def test_init_1d_sequence() -> None:
         [datetime(2020, 1, 1, tzinfo=timezone.utc)], schema={"ts": pl.Datetime("ms")}
     )
     assert df.schema == {"ts": pl.Datetime("ms", "UTC")}
-    with pytest.warns(
-        TimeZoneAwareConstructorWarning, match="Series with UTC time zone"
-    ):
-        df = pl.DataFrame(
-            [datetime(2020, 1, 1, tzinfo=timezone(timedelta(hours=1)))],
-            schema={"ts": pl.Datetime("ms")},
-        )
+    df = pl.DataFrame(
+        [datetime(2020, 1, 1, tzinfo=timezone(timedelta(hours=1)))],
+        schema={"ts": pl.Datetime("ms")},
+    )
     assert df.schema == {"ts": pl.Datetime("ms", "UTC")}
-    with pytest.warns(
-        TimeZoneAwareConstructorWarning, match="Series with UTC time zone"
-    ):
-        df = pl.DataFrame(
-            [datetime(2020, 1, 1, tzinfo=ZoneInfo("Asia/Kathmandu"))],
-            schema={"ts": pl.Datetime("ms")},
-        )
+    df = pl.DataFrame(
+        [datetime(2020, 1, 1, tzinfo=ZoneInfo("Asia/Kathmandu"))],
+        schema={"ts": pl.Datetime("ms")},
+    )
     assert df.schema == {"ts": pl.Datetime("ms", "UTC")}
 
 
@@ -957,17 +961,25 @@ def test_init_pandas(monkeypatch: Any) -> None:
 
     # pandas is not available
     monkeypatch.setattr(pl.dataframe.frame, "_check_for_pandas", lambda x: False)
-    with pytest.raises(TypeError):
-        pl.DataFrame(pandas_df)
+
+    # pandas 2.2 and higher implement the Arrow PyCapsule Interface, so the constructor
+    # will still work even without using pandas APIs
+    if parse_version(pd.__version__) >= parse_version("2.2.0"):
+        df = pl.DataFrame(pandas_df)
+        assert_frame_equal(df, expected)
+
+    else:
+        with pytest.raises(TypeError):
+            pl.DataFrame(pandas_df)
 
 
 def test_init_errors() -> None:
     # Length mismatch
-    with pytest.raises(pl.ShapeError):
+    with pytest.raises(ShapeError):
         pl.DataFrame({"a": [1, 2, 3], "b": [1.0, 2.0, 3.0, 4.0]})
 
     # Columns don't match data dimensions
-    with pytest.raises(pl.ShapeError):
+    with pytest.raises(ShapeError):
         pl.DataFrame([[1, 2], [3, 4]], schema=["a", "b", "c"])
 
     # Unmatched input
@@ -1060,11 +1072,9 @@ def test_init_only_columns() -> None:
             ],
         )
         expected = pl.DataFrame({"a": [], "b": [], "c": []}).with_columns(
-            [
-                pl.col("a").cast(pl.Date),
-                pl.col("b").cast(pl.UInt64),
-                pl.col("c").cast(pl.Int8),
-            ]
+            pl.col("a").cast(pl.Date),
+            pl.col("b").cast(pl.UInt64),
+            pl.col("c").cast(pl.Int8),
         )
         expected.insert_column(3, pl.Series("d", [], pl.List(pl.UInt8)))
 
@@ -1119,7 +1129,7 @@ def test_from_dicts_list_struct_without_inner_dtype_5611() -> None:
 
 
 def test_from_dict_upcast_primitive() -> None:
-    df = pl.from_dict({"a": [1, 2.1, 3], "b": [4, 5, 6.4]})
+    df = pl.from_dict({"a": [1, 2.1, 3], "b": [4, 5, 6.4]}, strict=False)
     assert df.dtypes == [pl.Float64, pl.Float64]
 
 
@@ -1303,7 +1313,7 @@ def test_from_records_nullable_structs() -> None:
         {"id": 1, "items": [{"item_id": 100, "description": "hi"}]},
     ]
 
-    schema: list[tuple[str, pl.PolarsDataType]] = [
+    schema: list[tuple[str, PolarsDataType]] = [
         ("id", pl.UInt16),
         (
             "items",
@@ -1315,7 +1325,7 @@ def test_from_records_nullable_structs() -> None:
         ),
     ]
 
-    schema_options: list[list[tuple[str, pl.PolarsDataType]] | None] = [schema, None]
+    schema_options: list[list[tuple[str, PolarsDataType]] | None] = [schema, None]
     for s in schema_options:
         result = pl.DataFrame(records, schema=s, orient="row")
         expected = {
@@ -1333,7 +1343,7 @@ def test_from_records_nullable_structs() -> None:
     assert df.to_dict(as_series=False) == {"id": [], "items": []}
     assert df.schema == dict_schema
 
-    dtype: pl.PolarsDataType = dict_schema["items"]
+    dtype: PolarsDataType = dict_schema["items"]
     series = pl.Series("items", dtype=dtype)
     assert series.to_frame().to_dict(as_series=False) == {"items": []}
     assert series.dtype == dict_schema["items"]
@@ -1624,7 +1634,116 @@ def test_array_construction() -> None:
         {"row_id": "a", "data": [1, 2, 3]},
         {"row_id": "b", "data": [2, 3, 4]},
     ]
-    schema = {"row_id": pl.String(), "data": pl.Array(inner=pl.Int64, width=3)}
+    schema = {"row_id": pl.String(), "data": pl.Array(inner=pl.Int64, shape=3)}
     df = pl.from_dicts(rows, schema=schema)
     assert df.schema == schema
     assert df.rows() == [("a", [1, 2, 3]), ("b", [2, 3, 4])]
+
+
+class PyCapsuleStreamHolder:
+    """
+    Hold the Arrow C Stream pycapsule.
+
+    A class that exposes _only_ the Arrow C Stream interface via Arrow PyCapsules. This
+    ensures that the consumer is seeing _only_ the `__arrow_c_stream__` dunder, and that
+    nothing else (e.g. the dataframe or array interface) is actually being used.
+    """
+
+    arrow_obj: Any
+
+    def __init__(self, arrow_obj: object) -> None:
+        self.arrow_obj = arrow_obj
+
+    def __arrow_c_stream__(self, requested_schema: object = None) -> object:
+        return self.arrow_obj.__arrow_c_stream__(requested_schema)
+
+
+class PyCapsuleArrayHolder:
+    """
+    Hold the Arrow C Array pycapsule.
+
+    A class that exposes _only_ the Arrow C Array interface via Arrow PyCapsules. This
+    ensures that the consumer is seeing _only_ the `__arrow_c_array__` dunder, and that
+    nothing else (e.g. the dataframe or array interface) is actually being used.
+    """
+
+    arrow_obj: Any
+
+    def __init__(self, arrow_obj: object) -> None:
+        self.arrow_obj = arrow_obj
+
+    def __arrow_c_array__(self, requested_schema: object = None) -> object:
+        return self.arrow_obj.__arrow_c_array__(requested_schema)
+
+
+def test_pycapsule_interface(df: pl.DataFrame) -> None:
+    pyarrow_table = df.to_arrow()
+
+    # Array via C data interface
+    pyarrow_array = pyarrow_table["bools"].chunk(0)
+    round_trip_series = pl.Series(PyCapsuleArrayHolder(pyarrow_array))
+    assert df["bools"].equals(round_trip_series, check_dtypes=True, check_names=False)
+
+    # empty Array via C data interface
+    empty_pyarrow_array = pa.array([], type=pyarrow_array.type)
+    round_trip_series = pl.Series(PyCapsuleArrayHolder(empty_pyarrow_array))
+    assert df["bools"].dtype == round_trip_series.dtype
+
+    # RecordBatch via C array interface
+    pyarrow_record_batch = pyarrow_table.to_batches()[0]
+    round_trip_df = pl.DataFrame(PyCapsuleArrayHolder(pyarrow_record_batch))
+    assert df.equals(round_trip_df)
+
+    # ChunkedArray via C stream interface
+    pyarrow_chunked_array = pyarrow_table["bools"]
+    round_trip_series = pl.Series(PyCapsuleStreamHolder(pyarrow_chunked_array))
+    assert df["bools"].equals(round_trip_series, check_dtypes=True, check_names=False)
+
+    # empty ChunkedArray via C stream interface
+    empty_chunked_array = pa.chunked_array([], type=pyarrow_chunked_array.type)
+    round_trip_series = pl.Series(PyCapsuleStreamHolder(empty_chunked_array))
+    assert df["bools"].dtype == round_trip_series.dtype
+
+    # Table via C stream interface
+    round_trip_df = pl.DataFrame(PyCapsuleStreamHolder(pyarrow_table))
+    assert df.equals(round_trip_df)
+
+    # empty Table via C stream interface
+    empty_df = df[:0].to_arrow()
+    round_trip_df = pl.DataFrame(PyCapsuleStreamHolder(empty_df))
+    orig_schema = df.schema
+    round_trip_schema = round_trip_df.schema
+
+    # The "enum" schema is not preserved because categories are lost via C data
+    # interface
+    orig_schema.pop("enum")
+    round_trip_schema.pop("enum")
+
+    assert orig_schema == round_trip_schema
+
+    # RecordBatchReader via C stream interface
+    pyarrow_reader = pa.RecordBatchReader.from_batches(
+        pyarrow_table.schema, pyarrow_table.to_batches()
+    )
+    round_trip_df = pl.DataFrame(PyCapsuleStreamHolder(pyarrow_reader))
+    assert df.equals(round_trip_df)
+
+
+@pytest.mark.parametrize(
+    "tz",
+    [
+        None,
+        ZoneInfo("Asia/Tokyo"),
+        ZoneInfo("Europe/Amsterdam"),
+        ZoneInfo("UTC"),
+        timezone.utc,
+    ],
+)
+def test_init_list_of_dicts_with_timezone(tz: Any) -> None:
+    dt = datetime(2023, 1, 1, 0, 0, 0, 0, tzinfo=tz)
+
+    df = pl.DataFrame([{"dt": dt}, {"dt": dt}])
+    expected = pl.DataFrame({"dt": [dt, dt]})
+    assert_frame_equal(df, expected)
+
+    assert df.schema == {"dt": pl.Datetime("us", time_zone=tz and "UTC")}
