@@ -11,16 +11,17 @@ import polars as pl
 from polars import StringCache
 from polars.exceptions import (
     CategoricalRemappingWarning,
+    ComputeError,
     StringCacheMismatchError,
 )
 from polars.testing import assert_frame_equal, assert_series_equal
 
 if TYPE_CHECKING:
-    from polars.type_aliases import PolarsDataType
+    from polars._typing import PolarsDataType
 
 
 @StringCache()
-def test_categorical_outer_join() -> None:
+def test_categorical_full_outer_join() -> None:
     df1 = pl.DataFrame(
         [
             pl.Series("key1", [42]),
@@ -49,7 +50,7 @@ def test_categorical_outer_join() -> None:
         schema_overrides={"key2": pl.Categorical, "key2_right": pl.Categorical},
     )
 
-    out = df1.join(df2, on=["key1", "key2"], how="outer").collect()
+    out = df1.join(df2, on=["key1", "key2"], how="full").collect()
     assert_frame_equal(out, expected)
 
     dfa = pl.DataFrame(
@@ -65,7 +66,7 @@ def test_categorical_outer_join() -> None:
         ]
     )
 
-    df = dfa.join(dfb, on="key", how="outer")
+    df = dfa.join(dfb, on="key", how="full")
     # the cast is important to test the rev map
     assert df["key"].cast(pl.String).to_list() == ["bar", None, "foo"]
     assert df["key_right"].cast(pl.String).to_list() == ["bar", "baz", None]
@@ -75,7 +76,7 @@ def test_read_csv_categorical() -> None:
     f = io.BytesIO()
     f.write(b"col1,col2,col3,col4,col5,col6\n'foo',2,3,4,5,6\n'bar',8,9,10,11,12")
     f.seek(0)
-    df = pl.read_csv(f, has_header=True, dtypes={"col1": pl.Categorical})
+    df = pl.read_csv(f, has_header=True, schema_overrides={"col1": pl.Categorical})
     assert df["col1"].dtype == pl.Categorical
 
 
@@ -401,7 +402,7 @@ def test_categorical_error_on_local_cmp() -> None:
 
 def test_cast_null_to_categorical() -> None:
     assert pl.DataFrame().with_columns(
-        [pl.lit(None).cast(pl.Categorical).alias("nullable_enum")]
+        pl.lit(None).cast(pl.Categorical).alias("nullable_enum")
     ).dtypes == [pl.Categorical]
 
 
@@ -472,7 +473,7 @@ def test_cast_inner_categorical() -> None:
     assert out.to_list() == [["a"], ["a", "b"]]
 
     with pytest.raises(
-        pl.ComputeError, match=r"casting to categorical not allowed in `list.eval`"
+        ComputeError, match=r"casting to categorical not allowed in `list.eval`"
     ):
         pl.Series("foo", [["a", "b"], ["a", "b"]]).list.eval(
             pl.element().cast(pl.Categorical)
@@ -485,7 +486,7 @@ def test_stringcache() -> None:
     with pl.StringCache():
         # create a large enough column that the categorical map is reallocated
         df = pl.DataFrame({"cats": pl.arange(0, N, eager=True)}).select(
-            [pl.col("cats").cast(pl.String).cast(pl.Categorical)]
+            pl.col("cats").cast(pl.String).cast(pl.Categorical)
         )
         assert df.filter(pl.col("cats").is_in(["1", "2"])).to_dict(as_series=False) == {
             "cats": ["1", "2"]
@@ -509,8 +510,8 @@ def test_categorical_sort_order_by_parameter(
 
 
 @StringCache()
-@pytest.mark.filterwarnings("ignore:`set_ordering` is deprecated:DeprecationWarning")
-def test_categorical_sort_order(monkeypatch: Any) -> None:
+@pytest.mark.parametrize("row_fmt_sort_enabled", [False, True])
+def test_categorical_sort_order(row_fmt_sort_enabled: bool, monkeypatch: Any) -> None:
     # create the categorical ordering first
     pl.Series(["foo", "bar", "baz"], dtype=pl.Categorical)
     df = pl.DataFrame(
@@ -521,18 +522,14 @@ def test_categorical_sort_order(monkeypatch: Any) -> None:
         }
     )
 
-    assert df.sort(["n", "x"])["x"].to_list() == ["foo", "bar", "baz"]
-    assert df.with_columns(pl.col("x").cat.set_ordering("lexical")).sort(["n", "x"])[
-        "x"
-    ].to_list() == ["bar", "baz", "foo"]
-    monkeypatch.setenv("POLARS_ROW_FMT_SORT", "1")
-    assert df.sort(["n", "x"])["x"].to_list() == ["foo", "bar", "baz"]
-    assert df.with_columns(pl.col("x").cat.set_ordering("lexical")).sort(["n", "x"])[
-        "x"
-    ].to_list() == ["bar", "baz", "foo"]
-    assert df.with_columns(pl.col("x").cast(pl.Categorical("lexical"))).sort(
-        ["n", "x"]
-    )["x"].to_list() == ["bar", "baz", "foo"]
+    if row_fmt_sort_enabled:
+        monkeypatch.setenv("POLARS_ROW_FMT_SORT", "1")
+
+    result = df.sort(["n", "x"])
+    assert result["x"].to_list() == ["foo", "bar", "baz"]
+
+    result = df.with_columns(pl.col("x").cast(pl.Categorical("lexical"))).sort("n", "x")
+    assert result["x"].to_list() == ["bar", "baz", "foo"]
 
 
 def test_err_on_categorical_asof_join_by_arg() -> None:
@@ -826,3 +823,25 @@ def test_cat_preserve_lexical_ordering_on_concat() -> None:
     df = pl.DataFrame({"x": ["b", "a", "c"]}).with_columns(pl.col("x").cast(dtype))
     df2 = pl.concat([df, df])
     assert df2["x"].dtype == dtype
+
+
+def test_cat_append_lexical_sorted_flag() -> None:
+    df = pl.DataFrame({"x": [0, 1, 1], "y": ["B", "B", "A"]}).with_columns(
+        pl.col("y").cast(pl.Categorical(ordering="lexical"))
+    )
+    df2 = pl.concat([part.sort("y") for part in df.partition_by("x")])
+
+    assert not (df2["y"].is_sorted())
+
+
+def test_get_cat_categories_multiple_chunks() -> None:
+    df = pl.DataFrame(
+        [
+            pl.Series("e", ["a", "b"], pl.Enum(["a", "b"])),
+        ]
+    )
+    df = pl.concat(
+        [df for _ in range(100)], how="vertical", rechunk=False, parallel=True
+    )
+    df_cat = df.lazy().select(pl.col("e").cat.get_categories()).collect()
+    assert len(df_cat) == 2
