@@ -3,7 +3,7 @@ from __future__ import annotations
 import io
 from datetime import datetime, time, timezone
 from decimal import Decimal
-from typing import TYPE_CHECKING, cast
+from typing import TYPE_CHECKING, Any, cast
 
 import fsspec
 import numpy as np
@@ -12,14 +12,17 @@ import pyarrow as pa
 import pyarrow.dataset as ds
 import pyarrow.parquet as pq
 import pytest
+from hypothesis import HealthCheck, given, settings
 
 import polars as pl
+from polars.exceptions import ComputeError
 from polars.testing import assert_frame_equal, assert_series_equal
+from polars.testing.parametric import dataframes
 
 if TYPE_CHECKING:
     from pathlib import Path
 
-    from polars.type_aliases import ParquetCompression
+    from polars._typing import ParquetCompression
     from tests.unit.conftest import MemoryUsage
 
 
@@ -42,10 +45,12 @@ COMPRESSIONS = [
 
 
 @pytest.mark.write_disk()
-def test_write_parquet_using_pyarrow_9753(tmpdir: Path) -> None:
+def test_write_parquet_using_pyarrow_9753(tmp_path: Path) -> None:
+    tmp_path.mkdir(exist_ok=True)
+
     df = pl.DataFrame({"a": [1, 2, 3]})
     df.write_parquet(
-        tmpdir / "test.parquet",
+        tmp_path / "test.parquet",
         compression="zstd",
         statistics=True,
         use_pyarrow=True,
@@ -89,25 +94,39 @@ def small_parquet_path(io_files_path: Path) -> Path:
 def test_to_from_buffer(
     df: pl.DataFrame, compression: ParquetCompression, use_pyarrow: bool
 ) -> None:
-    print(df)
     df = df[["list_str"]]
-    print(df)
     buf = io.BytesIO()
     df.write_parquet(buf, compression=compression, use_pyarrow=use_pyarrow)
     buf.seek(0)
     read_df = pl.read_parquet(buf, use_pyarrow=use_pyarrow)
-    print(read_df)
     assert_frame_equal(df, read_df, categorical_as_str=True)
 
 
+@pytest.mark.parametrize("use_pyarrow", [True, False])
+@pytest.mark.parametrize("rechunk_and_expected_chunks", [(True, 1), (False, 3)])
+def test_read_parquet_respects_rechunk_16416(
+    use_pyarrow: bool, rechunk_and_expected_chunks: tuple[bool, int]
+) -> None:
+    # Create a dataframe with 3 chunks:
+    df = pl.DataFrame({"a": [1]})
+    df = pl.concat([df, df, df])
+    buf = io.BytesIO()
+    df.write_parquet(buf, row_group_size=1)
+    buf.seek(0)
+
+    rechunk, expected_chunks = rechunk_and_expected_chunks
+    result = pl.read_parquet(buf, use_pyarrow=use_pyarrow, rechunk=rechunk)
+    assert result.n_chunks() == expected_chunks
+
+
 def test_to_from_buffer_lzo(df: pl.DataFrame) -> None:
     buf = io.BytesIO()
     # Writing lzo compressed parquet files is not supported for now.
-    with pytest.raises(pl.ComputeError):
+    with pytest.raises(ComputeError):
         df.write_parquet(buf, compression="lzo", use_pyarrow=False)
     buf.seek(0)
     # Invalid parquet file as writing failed.
-    with pytest.raises(pl.ComputeError):
+    with pytest.raises(ComputeError):
         _ = pl.read_parquet(buf)
 
     buf = io.BytesIO()
@@ -116,7 +135,7 @@ def test_to_from_buffer_lzo(df: pl.DataFrame) -> None:
         df.write_parquet(buf, compression="lzo", use_pyarrow=True)
     buf.seek(0)
     # Invalid parquet file as writing failed.
-    with pytest.raises(pl.ComputeError):
+    with pytest.raises(ComputeError):
         _ = pl.read_parquet(buf)
 
 
@@ -140,10 +159,10 @@ def test_to_from_file_lzo(df: pl.DataFrame, tmp_path: Path) -> None:
     file_path = tmp_path / "small.avro"
 
     # Writing lzo compressed parquet files is not supported for now.
-    with pytest.raises(pl.ComputeError):
+    with pytest.raises(ComputeError):
         df.write_parquet(file_path, compression="lzo", use_pyarrow=False)
     # Invalid parquet file as writing failed.
-    with pytest.raises(pl.ComputeError):
+    with pytest.raises(ComputeError):
         _ = pl.read_parquet(file_path)
 
     # Writing lzo compressed parquet files is not supported for now.
@@ -259,8 +278,10 @@ def test_lazy_self_join_file_cache_prop_3979(df: pl.DataFrame, tmp_path: Path) -
 
     a = pl.scan_parquet(file_path)
     b = pl.DataFrame({"a": [1]}).lazy()
-    assert a.join(b, how="cross").collect().shape == (3, df.width + b.width)
-    assert b.join(a, how="cross").collect().shape == (3, df.width + b.width)
+
+    expected_shape = (3, df.width + b.collect_schema().len())
+    assert a.join(b, how="cross").collect().shape == expected_shape
+    assert b.join(a, how="cross").collect().shape == expected_shape
 
 
 def test_recursive_logical_type() -> None:
@@ -414,8 +435,8 @@ def test_fetch_union(tmp_path: Path) -> None:
     df1.write_parquet(file_path_1)
     df2.write_parquet(file_path_2)
 
-    result_one = pl.scan_parquet(file_path_1).fetch(1)
-    result_glob = pl.scan_parquet(file_path_glob).fetch(1)
+    result_one = pl.scan_parquet(file_path_1)._fetch(1)
+    result_glob = pl.scan_parquet(file_path_glob)._fetch(1)
 
     expected = pl.DataFrame({"a": [0], "b": [1]})
     assert_frame_equal(result_one, expected)
@@ -554,7 +575,6 @@ def test_nested_struct_read_12610() -> None:
     assert_frame_equal(expect, actual)
 
 
-@pl.Config(activate_decimals=True)
 @pytest.mark.write_disk()
 def test_decimal_parquet(tmp_path: Path) -> None:
     path = tmp_path / "foo.parquet"
@@ -785,11 +805,117 @@ def test_sliced_dict_with_nulls_14904() -> None:
 
 
 def test_parquet_array_dtype() -> None:
-    df = pl.DataFrame({"x": [[1, 2, 3]]})
-    df = df.cast({"x": pl.Array(pl.Int64, width=3)})
+    df = pl.DataFrame({"x": []})
+    df = df.cast({"x": pl.Array(pl.Int64, shape=3)})
     test_round_trip(df)
 
 
+def test_parquet_array_dtype_nulls() -> None:
+    df = pl.DataFrame({"x": [[1, 2], None, [None, 3]]})
+    df = df.cast({"x": pl.Array(pl.Int64, shape=2)})
+    test_round_trip(df)
+
+
+@pytest.mark.parametrize(
+    ("series", "dtype"),
+    [
+        ([[1, 2, 3]], pl.List(pl.Int64)),
+        ([[1, None, 3]], pl.List(pl.Int64)),
+        (
+            [{"a": []}, {"a": [1]}, {"a": [1, 2, 3]}],
+            pl.Struct({"a": pl.List(pl.Int64)}),
+        ),
+        ([{"a": None}, None, {"a": [1, 2, None]}], pl.Struct({"a": pl.List(pl.Int64)})),
+        (
+            [[{"a": []}, {"a": [1]}, {"a": [1, 2, 3]}], None, [{"a": []}, {"a": [42]}]],
+            pl.List(pl.Struct({"a": pl.List(pl.Int64)})),
+        ),
+        (
+            [
+                [1, None, 3],
+                None,
+                [1, 3, 4],
+                None,
+                [9, None, 4],
+                [None, 42, 13],
+                [37, 511, None],
+            ],
+            pl.List(pl.Int64),
+        ),
+        ([[1, 2, 3]], pl.Array(pl.Int64, 3)),
+        ([[1, None, 3], None, [1, 2, None]], pl.Array(pl.Int64, 3)),
+        ([[1, 2], None, [None, 3]], pl.Array(pl.Int64, 2)),
+        # @TODO: Enable when zero-width arrays are enabled
+        # ([[], [], []],                       pl.Array(pl.Int64, 0)),
+        # ([[], None, []],                     pl.Array(pl.Int64, 0)),
+        (
+            [[[1, 5, 2], [42, 13, 37]], [[1, 2, 3], [5, 2, 3]], [[1, 2, 1], [3, 1, 3]]],
+            pl.Array(pl.Array(pl.Int8, 3), 2),
+        ),
+        (
+            [[[1, 5, 2], [42, 13, 37]], None, [None, [3, 1, 3]]],
+            pl.Array(pl.Array(pl.Int8, 3), 2),
+        ),
+        (
+            [
+                [[[2, 1], None, [4, 1], None], []],
+                None,
+                [None, [[4, 4], None, [1, 2]]],
+            ],
+            pl.Array(pl.List(pl.Array(pl.Int8, 2)), 2),
+        ),
+        ([[[], []]], pl.Array(pl.List(pl.Array(pl.Int8, 2)), 2)),
+        (
+            [
+                [
+                    [[[42, 13, 37, 15, 9, 20, 0, 0, 5, 10], None]],
+                    [None, [None, [1, 2, 3, 4, 5, 6, 7, 8, 9, 0]], None],
+                ]
+            ],
+            pl.Array(pl.List(pl.Array(pl.Array(pl.Int8, 10), 2)), 2),
+        ),
+        (
+            [
+                None,
+                [None],
+                [[None]],
+                [[[None]]],
+                [[[[None]]]],
+                [[[[[None]]]]],
+                [[[[[1]]]]],
+            ],
+            pl.Array(pl.Array(pl.Array(pl.Array(pl.Array(pl.Int8, 1), 1), 1), 1), 1),
+        ),
+        (
+            [
+                None,
+                [None],
+                [[]],
+                [[None]],
+                [[[None], None]],
+                [[[None], []]],
+                [[[[None]], [[[1]]]]],
+                [[[[[None]]]]],
+                [[[[[1]]]]],
+            ],
+            pl.Array(pl.List(pl.Array(pl.List(pl.Array(pl.Int8, 1)), 1)), 1),
+        ),
+    ],
+)
+@pytest.mark.write_disk()
+def test_complex_types(tmp_path: Path, series: list[Any], dtype: pl.DataType) -> None:
+    xs = pl.Series(series, dtype=dtype)
+    df = pl.DataFrame({"x": xs})
+
+    test_round_trip(df)
+
+
+@pytest.mark.xfail()
+def test_placeholder_zero_array() -> None:
+    # @TODO: if this does not fail anymore please enable the upper test-cases
+    pl.Series([[]], dtype=pl.Array(pl.Int8, 0))
+
+
 @pytest.mark.write_disk()
 def test_parquet_array_statistics(tmp_path: Path) -> None:
     tmp_path.mkdir(exist_ok=True)
@@ -835,3 +961,301 @@ def test_read_parquet_only_loads_selected_columns_15098(
     # Only one column's worth of memory should be used; 2 columns would be
     # 16_000_000 at least, but there's some overhead.
     assert 8_000_000 < memory_usage_without_pyarrow.get_peak() < 13_000_000
+
+
+@pytest.mark.release()
+@pytest.mark.write_disk()
+def test_max_statistic_parquet_writer(tmp_path: Path) -> None:
+    # this hits the maximal page size
+    # so the row group will be split into multiple pages
+    # the page statistics need to be correctly reduced
+    # for this query to make sense
+    n = 150_000
+
+    tmp_path.mkdir(exist_ok=True)
+
+    # int64 is important to hit the page size
+    df = pl.int_range(0, n, eager=True, dtype=pl.Int64).alias("int").to_frame()
+    f = tmp_path / "tmp.parquet"
+    df.write_parquet(f, statistics=True, use_pyarrow=False, row_group_size=n)
+    result = pl.scan_parquet(f).filter(pl.col("int") > n - 3).collect()
+    expected = pl.DataFrame({"int": [149998, 149999]})
+    assert_frame_equal(result, expected)
+
+
+@pytest.mark.slow()
+def test_hybrid_rle() -> None:
+    # 10_007 elements to test if not a nice multiple of 8
+    n = 10_007
+    literal_literal = []
+    literal_rle = []
+    for i in range(500):
+        literal_literal.append(np.repeat(i, 5))
+        literal_literal.append(np.repeat(i + 2, 11))
+        literal_rle.append(np.repeat(i, 5))
+        literal_rle.append(np.repeat(i + 2, 15))
+    literal_literal.append(np.random.randint(0, 10, size=2007))
+    literal_rle.append(np.random.randint(0, 10, size=7))
+    literal_literal = np.concatenate(literal_literal)
+    literal_rle = np.concatenate(literal_rle)
+    df = pl.DataFrame(
+        {
+            # Primitive types
+            "i64": pl.Series([1, 2], dtype=pl.Int64).sample(n, with_replacement=True),
+            "u64": pl.Series([1, 2], dtype=pl.UInt64).sample(n, with_replacement=True),
+            "i8": pl.Series([1, 2], dtype=pl.Int8).sample(n, with_replacement=True),
+            "u8": pl.Series([1, 2], dtype=pl.UInt8).sample(n, with_replacement=True),
+            "string": pl.Series(["abc", "def"], dtype=pl.String).sample(
+                n, with_replacement=True
+            ),
+            "categorical": pl.Series(["aaa", "bbb"], dtype=pl.Categorical).sample(
+                n, with_replacement=True
+            ),
+            # Fill up bit-packing buffer in middle of consecutive run
+            "large_bit_pack": np.concatenate(
+                [np.repeat(i, 5) for i in range(2000)]
+                + [np.random.randint(0, 10, size=7)]
+            ),
+            # Literal run that is not a multiple of 8 followed by consecutive
+            # run initially long enough to RLE but not after padding literal
+            "literal_literal": literal_literal,
+            # Literal run that is not a multiple of 8 followed by consecutive
+            # run long enough to RLE even after padding literal
+            "literal_rle": literal_rle,
+            # Final run not long enough to RLE
+            "final_literal": np.concatenate(
+                [np.random.randint(0, 100, 10_000), np.repeat(-1, 7)]
+            ),
+            # Final run long enough to RLE
+            "final_rle": np.concatenate(
+                [np.random.randint(0, 100, 9_998), np.repeat(-1, 9)]
+            ),
+            # Test filling up bit-packing buffer for encode_bool,
+            # which is only used to encode validities
+            "large_bit_pack_validity": [0, None] * 4092
+            + [0] * 9
+            + [1] * 9
+            + [2] * 10
+            + [0] * 1795,
+        }
+    )
+    f = io.BytesIO()
+    df.write_parquet(f)
+    f.seek(0)
+    for column in pq.ParquetFile(f).metadata.to_dict()["row_groups"][0]["columns"]:
+        assert "RLE_DICTIONARY" in column["encodings"]
+    f.seek(0)
+    assert_frame_equal(pl.read_parquet(f), df)
+
+
+@given(
+    df=dataframes(
+        allowed_dtypes=[
+            pl.Null,
+            pl.List,
+            pl.Array,
+            pl.Int8,
+            pl.UInt8,
+            pl.UInt32,
+            pl.Int64,
+            # pl.Date, # Turned off because of issue #17599
+            # pl.Time, # Turned off because of issue #17599
+            pl.Binary,
+            pl.Float32,
+            pl.Float64,
+            pl.String,
+            pl.Boolean,
+        ],
+        min_size=1,
+        max_size=5000,
+    )
+)
+@pytest.mark.slow()
+@pytest.mark.write_disk()
+@settings(suppress_health_check=[HealthCheck.function_scoped_fixture])
+def test_roundtrip_parametric(df: pl.DataFrame, tmp_path: Path) -> None:
+    # delete if exists
+    path = tmp_path / "data.parquet"
+
+    df.write_parquet(path)
+    result = pl.read_parquet(path)
+
+    assert_frame_equal(df, result)
+
+
+def test_parquet_statistics_uint64_16683() -> None:
+    u64_max = (1 << 64) - 1
+    df = pl.Series("a", [u64_max, 0], dtype=pl.UInt64).to_frame()
+    file = io.BytesIO()
+    df.write_parquet(file, statistics=True)
+    file.seek(0)
+    statistics = pq.read_metadata(file).row_group(0).column(0).statistics
+
+    assert statistics.min == 0
+    assert statistics.max == u64_max
+
+
+@pytest.mark.slow()
+@pytest.mark.parametrize("nullable", [True, False])
+def test_read_byte_stream_split(nullable: bool) -> None:
+    rng = np.random.default_rng(123)
+    num_rows = 1_000
+    values = rng.uniform(-1.0e6, 1.0e6, num_rows)
+    if nullable:
+        validity_mask = rng.integers(0, 2, num_rows).astype(np.bool_)
+    else:
+        validity_mask = None
+
+    schema = pa.schema(
+        [
+            pa.field("floats", type=pa.float32(), nullable=nullable),
+            pa.field("doubles", type=pa.float64(), nullable=nullable),
+        ]
+    )
+    arrays = [pa.array(values, type=field.type, mask=validity_mask) for field in schema]
+    table = pa.Table.from_arrays(arrays, schema=schema)
+    df = cast(pl.DataFrame, pl.from_arrow(table))
+
+    f = io.BytesIO()
+    pq.write_table(
+        table, f, compression="snappy", use_dictionary=False, use_byte_stream_split=True
+    )
+
+    f.seek(0)
+    read = pl.read_parquet(f)
+
+    assert_frame_equal(read, df)
+
+
+@pytest.mark.slow()
+@pytest.mark.parametrize("rows_nullable", [True, False])
+@pytest.mark.parametrize("item_nullable", [True, False])
+def test_read_byte_stream_split_arrays(
+    item_nullable: bool, rows_nullable: bool
+) -> None:
+    rng = np.random.default_rng(123)
+    num_rows = 1_000
+    max_array_len = 10
+    array_lengths = rng.integers(0, max_array_len + 1, num_rows)
+    if rows_nullable:
+        row_validity_mask = rng.integers(0, 2, num_rows).astype(np.bool_)
+        array_lengths[row_validity_mask] = 0
+        row_validity_mask = pa.array(row_validity_mask)
+    else:
+        row_validity_mask = None
+
+    offsets = np.zeros(num_rows + 1, dtype=np.int64)
+    np.cumsum(array_lengths, out=offsets[1:])
+    num_values = offsets[-1]
+    values = rng.uniform(-1.0e6, 1.0e6, num_values)
+
+    if item_nullable:
+        element_validity_mask = rng.integers(0, 2, num_values).astype(np.bool_)
+    else:
+        element_validity_mask = None
+
+    schema = pa.schema(
+        [
+            pa.field(
+                "floats",
+                type=pa.list_(pa.field("", pa.float32(), nullable=item_nullable)),
+                nullable=rows_nullable,
+            ),
+            pa.field(
+                "doubles",
+                type=pa.list_(pa.field("", pa.float64(), nullable=item_nullable)),
+                nullable=rows_nullable,
+            ),
+        ]
+    )
+    arrays = [
+        pa.ListArray.from_arrays(
+            pa.array(offsets),
+            pa.array(values, type=field.type.field(0).type, mask=element_validity_mask),
+            mask=row_validity_mask,
+        )
+        for field in schema
+    ]
+    table = pa.Table.from_arrays(arrays, schema=schema)
+    df = cast(pl.DataFrame, pl.from_arrow(table))
+
+    f = io.BytesIO()
+    pq.write_table(
+        table, f, compression="snappy", use_dictionary=False, use_byte_stream_split=True
+    )
+
+    f.seek(0)
+    read = pl.read_parquet(f)
+
+    assert_frame_equal(read, df)
+
+
+@pytest.mark.write_disk()
+def test_parquet_nested_null_array_17795(tmp_path: Path) -> None:
+    filename = tmp_path / "nested_null.parquet"
+
+    pl.DataFrame([{"struct": {"field": None}}]).write_parquet(filename)
+    pq.read_table(filename)
+
+
+@pytest.mark.write_disk()
+def test_parquet_record_batches_pyarrow_fixed_size_list_16614(tmp_path: Path) -> None:
+    filename = tmp_path / "a.parquet"
+
+    # @NOTE:
+    # The minimum that I could get it to crash which was ~132000, but let's
+    # just do 150000 to be sure.
+    n = 150000
+    x = pl.DataFrame(
+        {"x": np.linspace((1, 2), (2 * n, 2 * n * 1), n, dtype=np.float32)},
+        schema={"x": pl.Array(pl.Float32, 2)},
+    )
+
+    x.write_parquet(filename)
+    b = pl.read_parquet(filename, use_pyarrow=True)
+
+    assert b["x"].shape[0] == n
+    assert_frame_equal(b, x)
+
+
+@pytest.mark.write_disk()
+def test_parquet_list_element_field_name(tmp_path: Path) -> None:
+    filename = tmp_path / "list.parquet"
+
+    (
+        pl.DataFrame(
+            {
+                "a": [[1, 2], [1, 1, 1]],
+            },
+            schema={"a": pl.List(pl.Int64)},
+        ).write_parquet(filename, use_pyarrow=False)
+    )
+
+    schema_str = str(pq.read_schema(filename))
+    assert "<element: int64>" in schema_str
+    assert "child 0, element: int64" in schema_str
+
+
+@pytest.mark.parametrize(
+    ("s", "elem"),
+    [
+        (pl.Series(["", "hello", "hi", ""], dtype=pl.String), ""),
+        (pl.Series([0, 1, 2, 0], dtype=pl.Int64), 0),
+        (pl.Series([[0], [1], [2], [0]], dtype=pl.Array(pl.Int64, 1)), [0]),
+        (
+            pl.Series([[0, 1], [1, 2], [2, 3], [0, 1]], dtype=pl.Array(pl.Int64, 2)),
+            [0, 1],
+        ),
+    ],
+)
+def test_parquet_high_nested_null_17805(
+    s: pl.Series, elem: str | int | list[int]
+) -> None:
+    test_round_trip(
+        pl.DataFrame({"a": s}).select(
+            pl.when(pl.col("a") == elem)
+            .then(pl.lit(None))
+            .otherwise(pl.concat_list(pl.col("a").alias("b")))
+            .alias("c")
+        )
+    )
